extends Button

class_name ButtonAnimated

enum SelectionType {
	Mouse,
	Focus
}

export(NodePath) var animation_player_node
export(SelectionType) var interaction_type = SelectionType.Focus
export(String) var selected_animation
export(String) var unselected_animation
export(String) var clicked_animation
export(float,0,1,0.1) var click_delay = 0

onready var Anim:AnimationPlayer = get_node_or_null(animation_player_node)

signal pressed_direct

# Called when the node enters the scene tree for the first time.
func _ready():
	init_signals()

func init_signals():
	#print("Init signal on %s" %name)
	var select_signal = "mouse_entered" if interaction_type == SelectionType.Mouse else "focus_entered"
	var deselect_signal = "mouse_exited" if interaction_type == SelectionType.Mouse else "focus_exited"
	if interaction_type == SelectionType.Focus:
		connect("mouse_entered",self,"grab_focus")
	connect(select_signal,Anim,"play",[selected_animation])
	connect(deselect_signal,Anim,"play",[unselected_animation])
	if click_delay > 0:
		var connect_list = get_signal_connection_list("pressed")
		var DelayTimer = Timer.new()
		DelayTimer.wait_time = Anim.get_animation(clicked_animation).length * click_delay
		DelayTimer.one_shot = true
		DelayTimer.name = "DelayTimer"
		add_child(DelayTimer)
		for i in connect_list.size():
			#print(connect_list[i])
			DelayTimer.connect("timeout",connect_list[i].target,connect_list[i].method,connect_list[i].binds,connect_list[i].flags)
			disconnect("pressed",connect_list[i].target,connect_list[i].method)
		connect("pressed",DelayTimer,"start")
		connect("pressed",self,"emit_signal",["pressed_direct"])
	connect("pressed",Anim,"play",[clicked_animation])
