extends Node

#var min_vol
#var max_vol

var BGMData = []

onready var BGMPlayers = [$BGM,$BGM2]
onready var SFXPlayer = $SFX
onready var SFXLibrary = $SFXRes
onready var AudioTweener = $AudioTween

onready var crossfade_time = ProjectSettings.get_setting("vn/settings/sound/crossfade_time")
onready var min_vol = ProjectSettings.get_setting("vn/settings/sound/min_scale")
onready var max_vol = ProjectSettings.get_setting("vn/settings/sound/max_scale")

var sfx_priority = 0

func _ready():
	init_bgm_data()
	#pass

func init_bgm_data():
	var path = "res://Data/BGM/"
	var Dir = Directory.new()
	if Dir.open(path) == OK:
		Dir.list_dir_begin()
		var file_name = Dir.get_next()
		while file_name != "":
			if not Dir.current_is_dir():
				var file_path = path.plus_file(file_name)
				#print(file_path)
				if ResourceLoader.exists(file_path):
					var bgm_data = load(file_path)
					if bgm_data is DictionaryData:
						BGMData.append(bgm_data)
			file_name = Dir.get_next()

func set_volume(bus,value):
	#var scaled_vol = (value - get_min_val()) / get_max_vol()
	var scaled_vol = (value - min_vol) / max_vol
	print("%s scaled volume is %s" %[bus,scaled_vol])
	Func.set_audio_volume(bus,scaled_vol)

#func get_min_val():
#	if min_vol == null:
#		min_vol = ProjectSettings.get_setting("vn/settings/sound/min_scale")
#	return min_vol

#func get_max_vol():
#	if max_vol == null:
#		max_vol = ProjectSettings.get_setting("vn/settings/sound/max_scale")
#	return max_vol

func change_bgm(index, instant = false):
	if index < 0:
		stop_bgm(instant)
	else:
		print("Changing BGM to %s" %BGMData[index].data.title)
		VNData.unlock_bgm(index)
		if instant:
			play_current(index)
		else:
			AudioTweener.stop_all()
			if BGMPlayers[0].playing:
				BGMPlayers.invert()
				#Out and stop
				AudioTweener.interpolate_method(self,"set_previous_player_volume",1.0,0.0,crossfade_time,Tween.TRANS_LINEAR,Tween.EASE_IN)
				AudioTweener.interpolate_callback(BGMPlayers[1],crossfade_time,"stop")
			#play and in
			BGMPlayers[0].volume_db = linear2db(0)
			play_current(index)
			AudioTweener.interpolate_method(self,"set_current_player_volume",0.0,1.0,crossfade_time,Tween.TRANS_LINEAR,Tween.EASE_IN)
			AudioTweener.start()
		

func set_current_player_volume(value):
	BGMPlayers[0].volume_db = linear2db(value)

func set_previous_player_volume(value):
	BGMPlayers[1].volume_db = linear2db(value)

func play_current(index):
	BGMPlayers[0].stream = BGMData[index].data.stream
	BGMPlayers[0].play()

func pause_bgm():
	BGMPlayers[0].stream_paused = true

func resume_bgm():
	BGMPlayers[0].stream_paused = false

func stop_bgm(instant = false):
	print("Stopping BGM")
	if instant:
		BGMPlayers[0].stop()
	else:
		AudioTweener.interpolate_property(BGMPlayers[0],"volume_db",0,-60,crossfade_time,Tween.TRANS_LINEAR,Tween.EASE_IN)
		AudioTweener.interpolate_callback(BGMPlayers[0],crossfade_time,"stop")
		AudioTweener.start()

func play_sfx(name, priority = 0):
	if SFXLibrary.has_resource(name):
		#print(">> playing %s with priority %s" %[name,priority])
		if SFXPlayer.is_playing() and sfx_priority > priority:
			#print(">> returned %s with priority %s" %[name,priority])
			return
		sfx_priority = priority
		SFXPlayer.stream = SFXLibrary.get_resource(name)
		SFXPlayer.play()
