extends WindowControl

export(PackedScene) var ItemObj
export(NodePath) var CGViewerPath

signal cg_shown
signal cg_hidden

var CGView:CGViewer
var LastItem

onready var BackButton = $Margin/Panel/List/Main/Right/Back

# Called when the node enters the scene tree for the first time.
func _ready():
	init_cg_item()
	connect_viewer()

func connect_viewer():
	get_cg_viewer().connect("cg_closed",self,"cg_closed")

func get_cg_viewer() -> CGViewer:
	if CGView == null:
		CGView = get_node(CGViewerPath)
	return CGView

func init_cg_item():
	var grid_size = Func.get_cg_gallery_grid_size()
	var Grid:GridContainer = get_item_parent()
	Grid.columns = int(grid_size.x)
	var FirstCG
	var LastCG
	for i in range(int(grid_size.x * grid_size.y)):
		if i < VNAssets.CGData.size():
			var NewCGItem = ItemObj.instance()
			NewCGItem.get_node("Margin/Thumb").texture = VNAssets.CGData[i].data["thumbnail"]
			NewCGItem.connect("pressed",self,"show_cg",[i])
			NewCGItem.name = "CG%s" %i
			Grid.add_child(NewCGItem)
			if FirstCG == null:
				FirstCG = NewCGItem
			LastCG = NewCGItem
		else:
			var NewEmptyItem = ColorRect.new()
			NewEmptyItem.color = ColorN("white",0)
			NewEmptyItem.size_flags_horizontal = NewEmptyItem.SIZE_EXPAND_FILL
			NewEmptyItem.size_flags_vertical = NewEmptyItem.SIZE_EXPAND_FILL
			NewEmptyItem.name = "Empty%s" %i
			Grid.add_child(NewEmptyItem)
	if FirstCG != null:
		var FirstCGPath = Grid.get_path_to(FirstCG)
		Grid.focus_neighbour_right = FirstCGPath
		Grid.focus_neighbour_bottom = FirstCGPath
	if LastCG != null:
		var LastCGPath = Grid.get_path_to(LastCG)
		Grid.focus_neighbour_left = LastCGPath
		Grid.focus_neighbour_top = LastCGPath
	update_cg_state()

func update_cg_state():
	var active_states = VNData.get_cg_states()
	active_states.resize(VNAssets.CGData.size())
	for i in active_states.size():
		var current_state = active_states[i]
		if current_state == null:
			current_state = false
		var CurrentCG = get_item_parent().get_node("CG%s" %i) as ButtonAnimated
		var CGImageP = CurrentCG.get_node("Margin")
		var EPanel = CurrentCG.get_node("EPanel")
		CurrentCG.disabled = !current_state
		CGImageP.set_visible(current_state)
		EPanel.set_visible(!current_state)

func show_cg(index):
	LastItem = get_item_parent().get_focus_owner()
	LastItem.release_focus()
	BackButton.disabled = true
	emit_signal("cg_shown")
	get_cg_viewer().show_cg(index)

func cg_closed():
	if LastItem != null:
		LastItem.grab_focus()
	emit_signal("cg_hidden")
	BackButton.disabled = false
