extends Node

class_name MusicItem

var AudioPlayer

var playing = false
var track_stream = null
var music_id = -1
var unlocked = false

onready var Tweener:Tween = $Tween
onready var Seek:TextureProgress = $Seek
onready var ArtDefault:Texture = get_node_and_resource(NodePath("TextureButton:texture_normal"))[1]

signal music_played

func set_id(id,lock_state):
	music_id = id
	unlocked = lock_state
	track_stream = Audio.BGMData[id].data.stream
	#print("Music item, name = %s, id = %s, unlocked = %s" %[name,music_id,lock_state])
	
	var TrackNumber = $Labels/List/Track
	var TrackNumber2 = $Labels2/List/Track
	
	TrackNumber.set_text("Track%s" %(id+1))
	TrackNumber2.set_text("Track%s" %(id+1))
	
	update_visual()

func set_active_state(value):
	unlocked = value

func update_visual():
	var TrackTitle = $Labels/List/Title
	var TrackLength = $Labels/List/Length
	var TrackButton = $TextureButton
	var TrackTitle2 = $Labels2/List/Title
	var TrackLength2 = $Labels2/List/Length
	
	var title = "???" if not unlocked else Audio.BGMData[music_id].data.title
	var b_texture = ArtDefault if not unlocked else Audio.BGMData[music_id].data.art
	var length_str = "??:??"
	if unlocked:
		var track_sec = track_stream.get_length()
		var track_min = int(track_sec / 60)
		track_sec = int(track_sec) % 60
		length_str = "%02d:%02d" %[track_min,track_sec]
	
	TrackTitle.set_text(title)
	TrackTitle2.set_text(title)
	TrackLength.set_text(length_str)
	TrackLength2.set_text(length_str)
	
	if b_texture:
		#print("ArtDefault = %s" %ArtDefault)
		TrackButton.texture_normal = b_texture

func set_audio_player(audio_player):
	AudioPlayer = audio_player

func play(loop = false):
	if not unlocked:
		return
	emit_signal("music_played")
	if loop:
		Tweener.stop_all()
		var start_val = 0.0
		var end_val = 100.0
		var fill_mode = TextureProgress.FILL_CLOCKWISE
		if Seek.fill_mode == TextureProgress.FILL_CLOCKWISE:
			start_val = 100.0
			end_val = 0.0
			fill_mode = TextureProgress.FILL_COUNTER_CLOCKWISE
		Seek.value = start_val
		Seek.fill_mode = fill_mode
		print("Player seek tween loop -> track length %s" %track_stream.get_length())
		Tweener.interpolate_property(Seek,"value",start_val,end_val,track_stream.get_length(),Tween.TRANS_LINEAR,Tween.EASE_IN)
		Tweener.interpolate_callback(self,track_stream.get_length(),"play",true)
		Tweener.start()
	else:
		if playing:
			return
		get_tree().call_group("MusicItem","stop",music_id)
		Seek.value = 0
		Seek.fill_mode = TextureProgress.FILL_CLOCKWISE
		Tweener.stop_all()
		print("Player seek tween start -> track length %s" %track_stream.get_length())
		Tweener.interpolate_property(Seek,"self_modulate",ColorN("white",0.0),ColorN("white",0.5),0.3,Tween.TRANS_LINEAR,Tween.EASE_IN)
		Tweener.interpolate_property(Seek,"value",0.0,100.0,track_stream.get_length(),Tween.TRANS_LINEAR,Tween.EASE_IN)
		Tweener.interpolate_callback(self,track_stream.get_length(),"play",true)
		Tweener.start()
		AudioPlayer.stream = track_stream
		AudioPlayer.play()
		playing = true

func stop(id = -1):
	if not playing or id == music_id:
		return
	print("%s Stopping play" %name)
	Tweener.stop_all()
	Tweener.interpolate_property(Seek,"self_modulate",ColorN("white",0.5),ColorN("white",0.0),0.3,Tween.TRANS_LINEAR,Tween.EASE_IN)
	Tweener.start()
	playing = false

func get_class():
	return "MusicItem"
