extends WindowControl

var option = {
	"volume_main" : 100.0,
	"volume_bgm" : 60.0,
	"volume_sfx" : 60.0,
	"volume_voice" : 60.0,
	"display" : 1,
	"window_border" : 0,
	"frame_rate": 1,
	"resolution" : 0,
	"language" : 0
}

var new_settings = true
var resolutions = []
var lang_codes = []
var lang_names = []
var fullscreen_size:Vector2
var aspect_scale:float
var adapted_size:Vector2
var cursor_ref = null
var fps = [30,60]

func _ready():
	init_screen_size()
	init_res()
	init_langs()
	load_setting()
	init_value()

func init_screen_size():
	var size_override = Func.set_screen_size_override()
	fullscreen_size = OS.get_screen_size() if size_override == Vector2.ZERO else size_override
	aspect_scale = fullscreen_size.aspect() / Func.get_reference_size().aspect()
	adapted_size = Func.get_reference_size()
	if aspect_scale < 1:
		#4:3,etc -> side crop
		adapted_size.x *= aspect_scale
	else:
		#2:1,etx -> top bottom crop
		adapted_size.y *= (Func.get_reference_size().aspect() / fullscreen_size.aspect())
	print("Aspect scale is %s" %aspect_scale)
	print("Adapted size is %s" %adapted_size)

func init_res():
	var current_res = fullscreen_size
	var screen_scales = ProjectSettings.get_setting("vn/settings/option/resolution")
	resolutions.resize(screen_scales.size())
	for i in range(screen_scales.size()):
		resolutions[i] = (current_res * float(screen_scales[i])).round()
	#print(resolutions)

func init_langs():
	var path = "res://Data/Langs/"
	var Dir = Directory.new()
	#var lang_names = []
	if Dir.open(path) == OK:
		Dir.list_dir_begin()
		var file_name = Dir.get_next()
		while file_name != "":
			if not Dir.current_is_dir():
				var file_path = path.plus_file(file_name)
				#print(file_path)
				if ResourceLoader.exists(file_path):
					var lang_data = load(file_path)
					if lang_data is DictionaryData:
						lang_codes.append(lang_data.data.id)
						lang_names.append(lang_data.data.name)
			file_name = Dir.get_next()
	#lang_codes = ProjectSettings.get_setting("vn/settings/option/language_code") 
	ProjectSettings.set_setting("vn/settings/option/language", PoolStringArray(lang_names)) 
	ProjectSettings.set_setting("vn/settings/option/language_code", PoolStringArray(lang_codes)) 

func load_setting():
	var file_path = ProjectSettings.get_setting("vn/files/setting/path")
	var file_pepper = ProjectSettings.get_setting("vn/files/setting/pepper")
	var SettingFile = File.new()
	if SettingFile.file_exists(file_path):
		new_settings = false
		SettingFile.open_encrypted_with_pass(file_path,File.READ,file_pepper)
		var content = SettingFile.get_var()
		if content is Dictionary:
			for key in option.keys():
				if content.has(key):
					option[key] = content[key]
		SettingFile.close()

func save_settings():
	var file_path = ProjectSettings.get_setting("vn/files/setting/path")
	var file_pepper = ProjectSettings.get_setting("vn/files/setting/pepper")
	var SettingFile = File.new()
	SettingFile.open_encrypted_with_pass(file_path,File.WRITE,file_pepper)
	SettingFile.store_var(option)
	SettingFile.close()

func init_value():
	for key in option.keys():
		var KeyObj = get_item_parent().get_node_or_null(key)
		if KeyObj:
			if KeyObj.has_method("init_options"):
				KeyObj.call("init_options")
			KeyObj.set_value(option[key])
		if has_method("update_%s" %key):
			call("update_%s" %key)

func set_option(oname,value):
	if option.has(oname):
		option[oname] = value
	if has_method("update_%s" %oname):
		call("update_%s" %oname)

func update_lang_ui():
	var KeyObj = get_item_parent().get_node_or_null("language")
	if KeyObj:
		KeyObj.set_value(option["language"])

func update_volume_main():
	Audio.set_volume("Master",option.volume_main)

func update_volume_bgm():
	Audio.set_volume("BGM",option.volume_bgm)

func update_volume_sfx():
	Audio.set_volume("SFX",option.volume_sfx)

func update_volume_voice():
	Audio.set_volume("Voice",option.volume_voice)

func update_frame_rate():
	var target = fps[option.frame_rate]
	print("Target FPS is %s FPS" %target)
	Engine.target_fps = target

func update_window_border():
	OS.window_borderless = bool(option.window_border)
	OS.center_window()
	update_display()

func update_display():
	#return
	var current_state = !bool(option.display)
	OS.window_fullscreen = current_state
	if not current_state:
		update_resolution()
	elif OS.window_borderless:
		OS.window_fullscreen = false
		set_resolution(OS.get_screen_size())

func update_resolution():
	if not bool(option.display):
		return
	var selected_size = resolutions[option.resolution]
	set_resolution(selected_size)

func set_resolution(resolution):
	update_cursor(resolution)
	OS.window_size = resolution
	OS.center_window()

func update_language():
	var current_lang = lang_codes[option.language]
	#print("Current lang is %s" %current_lang)
	TranslationServer.set_locale(current_lang)

func update_cursor(resolution):
	var ref_height = ProjectSettings.get_setting("display/window/size/height")
	var cur_height = resolution.y
	var cursor_scale = stepify(cur_height / ref_height,0.01)
	if cursor_ref != cursor_scale:
		var cursor_texture:StreamTexture = load(ProjectSettings.get_setting("display/mouse_cursor/custom_image"))
		if cursor_texture == null:
			return
		var CursorImageData:Image = cursor_texture.get_data()
		var scale_size = cursor_texture.get_size() * cursor_scale
		CursorImageData.resize(int(scale_size.x),int(scale_size.y),Image.INTERPOLATE_LANCZOS)
		var ScaledTexture = ImageTexture.new()
		ScaledTexture.create_from_image(CursorImageData)
		Input.set_custom_mouse_cursor(ScaledTexture)
		cursor_ref = cursor_scale
		print("Cursor Scaled!")

