extends Reference

class_name Route

enum NodeType {
	Dialog = 0,
	Choice = 1,
	End = 2,
	ASD = 3,
	Operation = 4
}

var RouteData:DictionaryData
var current_node = ""
var history = PoolStringArray([])
var params = {}

signal advanced
signal choice_needed(index)
signal ended

func _init():
	var route_dpath = ProjectSettings.get_setting("vn/gameplay/data/route")
	RouteData = load(route_dpath)
	if RouteData.data.has("variables"):
		for variable in RouteData.data.variables:
			params[variable] = 0
	current_node = "Start"
	history.append(current_node)

func resume(dialog_id,choice):
	var target_node = get_dialog_node_contain_id(dialog_id)
	while current_node != target_node:
		advance()
		if get_node_type(current_node) == NodeType.Choice:
			select_choice(choice[current_node])

func advance():
	var next_nodes = get_next_nodes(current_node)
	for node in next_nodes:
		process_node(node)

func process_node(node_name):
	match RouteData.data.nodes[node_name].type:
		NodeType.Dialog:
			current_node = node_name
			history.append(current_node)
			print("Advanced to dialog node %s" %current_node)
			emit_signal("advanced")
		NodeType.Choice:
			current_node = node_name
			history.append(current_node)
			emit_signal("choice_needed",int(get_node_value(node_name)))
			#var choice_id = int(RouteData.data.nodes[node_name].value)
			#Choice.connect("selected",self,"select_choice",[node_name])
			#Choice.choice_open(choice_id)
		NodeType.Operation:
			execute_expression(get_node_value(node_name))
		NodeType.End:
			emit_signal("ended")

func select_choice(answer):
	if get_node_type(current_node) != NodeType.Choice:
		return
	var next_nodes = []
	for i in range(RouteData.data.connections.size()):
		if RouteData.data.connections[i].from == current_node:
			if RouteData.data.connections[i].from_port == answer:
				next_nodes.append(RouteData.data.connections[i].to)
	for node in next_nodes:
		process_node(node)
	

func execute_expression(expr):
	var operand = ""
	for variable in params.keys():
		if variable.is_subsequence_of(expr):
			operand = variable
			break
	if operand == "":
		return
	var expression = Expression.new()
	expression.parse(expr,[operand])
	params[operand] = expression.execute([params[operand]])

func get_next_nodes(prev_node) -> PoolStringArray:
	var next_nodes = PoolStringArray([])
	for i in range(RouteData.data.connections.size()):
		if RouteData.data.connections[i].from == prev_node:
			next_nodes.append(RouteData.data.connections[i].to)
	print("next node is %s" %next_nodes)
	return next_nodes

func create_log(current_id,current_choice,max_count):
	var log_arr = []
	var history_id = history.size() - 1
	var current_index = current_id
	var c_choice = current_choice.duplicate()
	var c_node = current_node
	var c_range = get_node_value(c_node)
	if not Func.is_inside_range(current_index,c_range):
		return log_arr
	for _i in range(max_count):
		current_index -= 1
		if current_index < 0:
			break
		if not Func.is_inside_range(current_index,c_range):
			history_id -= 1
			if history[history_id] == "Start":
				return log_arr
			while get_node_type(history[history_id]) != NodeType.Dialog:
				if c_choice.has(history[history_id]):
					c_choice.erase(history[history_id])
				history_id -= 1
				if history[history_id] == "Start":
					return log_arr
			c_node = history[history_id]
			c_range = get_node_value(c_node)
			current_index = int(c_range.y)
		var log_dict = {}
		log_dict["id"] = current_index
		log_dict["dialog"] = "dialog_%s" %current_index
		log_dict["choice"] = c_choice
		
		log_arr.append(log_dict)
	return log_arr

func get_dialog_ids_inside_range(dialog_range,current_dialog):
	var ids = []
	var current_range = get_current_node_value()
	var current_r = Vector2(0,current_dialog)
	if Func.is_inside_range(current_range.x,dialog_range):
		current_r.x = current_range.x
	else:
		current_r.x = dialog_range.x
	ids.append(current_r)
	var id = history.size() - 1
	id -= 1
	while history[id] != "Start":
		var temp_range = Vector2.ZERO
		if get_node_type(history[id]) == NodeType.Dialog:
			current_range = get_node_value(history[id])
			if Func.is_inside_range(current_range.y,dialog_range):
				if Func.is_inside_range(current_range.x,dialog_range):
					
					temp_range = current_range
				else:
					temp_range.y = current_range.y
					temp_range.x = dialog_range.x
			else:
				if Func.is_inside_range(current_range.x,dialog_range):
					temp_range.y = dialog_range.y
					temp_range.x = current_range.y
		if temp_range != Vector2.ZERO:
			ids.push_front(temp_range)
		id -= 1
	#print(">> Lookup range is -> %s" %var2str(ids))
	return ids

func get_dialog_node_contain_id(dialog_id):
	for node in RouteData.data.nodes.keys():
		if get_node_type(node) == NodeType.Dialog:
			if Func.is_inside_range(dialog_id,get_node_value(node)):
				return node
	return ""

func get_current_node_value():
	return get_node_value(current_node)

func get_current_node_tyoe():
	return get_node_type(current_node)

func get_node_type(node_name):
	return RouteData.data.nodes[node_name].type

func get_node_value(node_name):
	return RouteData.data.nodes[node_name].value
