extends WindowControl

enum Mode {
	Save,
	Load
}

export(PackedScene) var SaveSlotItem

var title = ["title_save", "title_load"]

var current_mode
var max_page
var last_slot = null
var last_page = null
var quick_data = null
var loaded_data = null
var sys_data = null
var confirmation = true

signal title_changed(title)
signal slider_changed(value)
signal page_updated(val_str)
signal max_page_updated(max_value_str)
signal max_page_changed(max_value)

signal game_loaded

func _ready():
	init_slot()
	init_sys_data()
	goto_page(sys_data.data.current_page)
	#reload_page_slots()

func init_slot():
	var Grid:GridContainer = get_item_parent()
	var grid_size = Func.get_save_slot_grid_size()
	max_page = Func.get_save_slot_page_count()
	emit_signal("max_page_changed",max_page)
	emit_signal("max_page_updated",str(max_page))
	var FirstSlot
	var LastSlot
	for i in range(int(grid_size.x * grid_size.y)):
		var NewSlot:ButtonAnimated = SaveSlotItem.instance()
		NewSlot.connect("pressed",self,"slot_clicked",[i])
		#print("Instantiated %s" %i)
		NewSlot.name = "Slot%s" %i
		Grid.add_child(NewSlot)
		update_slot_item(NewSlot,null)
		#NewSlot.connect("pressed")
		if FirstSlot == null:
			FirstSlot = NewSlot
		LastSlot = NewSlot
	if FirstSlot != null:
		var FirstPath = Grid.get_path_to(FirstSlot)
		Grid.focus_neighbour_right = FirstPath
		Grid.focus_neighbour_bottom = FirstPath
	if LastSlot != null:
		var LastPath = Grid.get_path_to(LastSlot)
		Grid.focus_neighbour_left = LastPath
		Grid.focus_neighbour_top = LastPath

func init_sys_data():
	var sys_path = Func.get_file_path("system")
	var sys_pepper = Func.get_file_pepper("system")
	var SysFile = File.new()
	if SysFile.file_exists(sys_path):
		SysFile.open_encrypted_with_pass(sys_path,File.READ,sys_pepper)
		print(">> Sys Data Content Before Read -> %s" %var2str(sys_data))
		sys_data = SysFile.get_var(true)
		#sys_data = sys_data as DictionaryData
		print(">> Sys Data Content After Read -> %s" %var2str(sys_data))
		SysFile.close()
	else:
		SysFile.close()
		sys_data = DictionaryData.new()
		sys_data.data.current_page = int(1)
		sys_data.data.last_slot = int(0)
		sys_data.data.readed_line = []
		sys_data.data.selected_choice = {}
		sys_data.data.endings = []
		save_sys_data()

func get_bgm_states() -> Array:
	if sys_data != null:
		if sys_data.data.has("bgm"):
			return sys_data.data.bgm
	return []

func unlock_bgm(index):
	if sys_data == null:
		return
	var changed = false
	if not sys_data.data.has("bgm"):
		changed = true
		sys_data.data.bgm = []
	if index >= sys_data.data.bgm.size():
		changed = true
		sys_data.data.bgm.resize(index + 1)
	if not sys_data.data.bgm[index]:
		changed = true
		sys_data.data.bgm[index] = true
	if changed:
		save_sys_data()

func get_cg_states() -> Array:
	if sys_data != null:
		if sys_data.data.has("cg"):
			return sys_data.data.cg
	return []

func unlock_cg(index):
	if sys_data == null:
		return
	var changed = false
	if not sys_data.data.has("cg"):
		changed = true
		sys_data.data.cg = []
	if index >= sys_data.data.cg.size():
		changed = true
		sys_data.data.cg.resize(index + 1)
	if not sys_data.data.cg[index]:
		changed = true
		sys_data.data.cg[index] = true
	if changed:
		save_sys_data()

func change_last_slot(index):
	if sys_data.data.last_slot != int(index):
		sys_data.data.last_slot = int(index)
		save_sys_data()

func change_last_page(index):
	if sys_data.data.current_page != int(index):
		sys_data.data.current_page = int(index)
		#save_sys_data()

func save_sys_data():
	var sys_path = Func.get_file_path("system")
	var sys_pepper = Func.get_file_pepper("system")
	var SysFile = File.new()
	SysFile.open_encrypted_with_pass(sys_path,File.WRITE,sys_pepper)
	SysFile.store_var(sys_data,true)
	SysFile.close()

func open(mode = Mode.Load):
	current_mode = mode
	emit_signal("title_changed",title[mode])
	#emit_signal("slider_changed",sys_data.data.current_page)
	#get_item_parent().get_node("Slot%s")
	update_page()
	.open()
	focus_last_item()

func focus_last_item():
	var LastItem = get_item_parent().get_node_or_null("Slot%s" %sys_data.data.last_slot)
	if LastItem:
		LastItem.grab_focus()

func navigate_page(addition):
	if typeof(addition) == TYPE_BOOL:
		if addition:
			#sys_data.data.current_page = max_page
			last_page = max_page
		else:
			#sys_data.data.current_page = 1.0
			last_page = 1.0
	else:
		#sys_data.data.current_page = Func.cycle(sys_data.data.current_page,addition,1.0,max_page)
		last_page = Func.cycle(last_page,addition,1.0,max_page)
	#emit_signal("slider_changed",sys_data.data.current_page)
	emit_signal("slider_changed",last_page)
	#update_page()
	#goto_page(sys_data.data.current_page)
	goto_page(last_page)

func goto_page(page):
	#sys_data.data.current_page = page
	last_page = page
	#emit_signal("slider_changed",sys_data.data.current_page)
	emit_signal("slider_changed",last_page)
	update_page()
	reload_page_slots()

func update_slot_item(SlotObj,data):
	var BigNode = SlotObj.get_node("Panel/Big")
	var SmallNode = SlotObj.get_node("Panel/Small")
	BigNode.visible = false if data == null else true
	SmallNode.visible = false if data == null else true
	#SlotObj.disabled = true if data == null else false
	if data != null:
		var ThumbNode = SlotObj.get_node("Panel/Big/Scaler/Thumb")
		var ChapterNode = SlotObj.get_node("Panel/Big/Scaler/List/Chapter")
		var ChapterNode2 = SlotObj.get_node("Panel/Small/Scaler/List/Chapter")
		var ChapterTitleNode = SlotObj.get_node("Panel/Big/Scaler/List/ChapterTitle")
		var ChapterTitleNode2 = SlotObj.get_node("Panel/Small/Scaler/List/ChapterTitle")
		var DateNode = SlotObj.get_node("Panel/Big/Scaler/List/Date")
		var DateNode2 = SlotObj.get_node("Panel/Small/Scaler/List/Date")
		var TimeNode = SlotObj.get_node("Panel/Big/Scaler/List/Time")
		var TimeNode2 = SlotObj.get_node("Panel/Small/Scaler/List/Time")
		var ThumbTexture = ImageTexture.new()
		var ImageData:Image = data.thumb
		ImageData.decompress()
		ThumbTexture.create_from_image(ImageData)
		var datetime_formated = data.datetime
		var dig2_format = ["month","day","hour","minute"]
		for key in dig2_format:
			datetime_formated[key] = "%02d" %datetime_formated[key]
		var chapter_idx = "Chapter %s" %(data.chapter + 1)
		var date = "{year}/{month}/{day}".format(datetime_formated)
		var time = "{hour}:{minute}".format(datetime_formated)
		ThumbNode.texture = ThumbTexture
		ChapterNode.set_text(chapter_idx)
		ChapterNode2.set_text(chapter_idx)
		ChapterTitleNode.set_text("chapter_title_%s" %data.chapter)
		ChapterTitleNode2.set_text("chapter_title_%s" %data.chapter)
		DateNode.set_text(date)
		DateNode2.set_text(date)
		TimeNode.set_text(time)
		TimeNode2.set_text(time)

func update_page():
	#emit_signal("page_updated",str(sys_data.data.current_page))
	emit_signal("page_updated",str(last_page))

func reload_page_slots():
	var slot_count = get_item_parent().get_child_count()
	#var start_index = (sys_data.data.current_page - 1) * slot_count
	#print("Reload slots, page = %s, start index = %s, count = %s" %[sys_data.data.current_page,start_index,slot_count])
	var start_index = (last_page - 1) * slot_count
	print("Reload slots, page = %s, start index = %s, count = %s" %[last_page,start_index,slot_count])
	for i in get_item_parent().get_child_count():
		var SlotNode = get_item_parent().get_child(i)
		var SlotFile = File.new()
		var file_path = Func.get_file_path("data") %(start_index+i)
		var file_data = null
		#print("Checking file %s" %file_path)
		if SlotFile.file_exists(file_path):
			var file_pepper = Func.get_file_pepper("data")
			SlotFile.open_encrypted_with_pass(file_path,File.READ,file_pepper)
			file_data = SlotFile.get_var(true)
			file_data = file_data.data
			SlotFile.close()
		else:
			SlotFile.close()
		update_slot_item(SlotNode,file_data)

func slot_clicked(slot_index):
	#calculate index
	last_slot = slot_index
	#var start_index = get_item_parent().get_child_count() * (sys_data.data.current_page - 1)
	var start_index = get_item_parent().get_child_count() * (last_page - 1)
	var SlotNode = get_item_parent().get_child(slot_index)
	slot_index = start_index + slot_index
	var save_path:String = Func.get_file_path("data") %slot_index
	var SaveFile = File.new()
	if SaveFile.file_exists(save_path):
		Audio.play_sfx("ui_accept")
		match current_mode:
			Mode.Save:
				Confirmation.connect("answered",self,"confirm_save",[save_path,SlotNode],CONNECT_ONESHOT)
				Confirmation.open("conf_owrite")
			Mode.Load:
				load_file(save_path)
	else:
		if current_mode == Mode.Save:
			Audio.play_sfx("ui_accept")
			confirm_save(true,save_path,SlotNode)
	SaveFile.close()

#func load_file(file_path):
#	print("Try to load %s" %file_path)

func load_file(file_path = ""):
	#print("Load game!")
	#return
	if file_path != "":
		if confirmation:
			Confirmation.connect("answered",self,"confirm_load",[file_path],CONNECT_ONESHOT)
			Confirmation.open("conf_load")
		else:
			confirm_load(true,file_path)
	

func confirm_load(answer,file_path):
	if answer:
		#Audio.play_sfx("ui_accept")
		var save_pepper:String = Func.get_file_pepper("data")
		var SaveFile = File.new()
		if SaveFile.file_exists(file_path):
			SaveFile.open_encrypted_with_pass(file_path,File.READ,save_pepper)
			#loaded_data = SaveFile.get_var(true) as DictionaryData
			var saved_data = SaveFile.get_var(true)
			loaded_data = DictionaryData.new()
			loaded_data.data = saved_data.data
			SaveFile.close()
			print("Load data saved at -> %s" %loaded_data.data.datetime)
			change_last_page(last_page)
			change_last_slot(last_slot)
			#print("> Game Loaded connection -> %s" %var2str(get_signal_connection_list("game_loaded")))
			emit_signal("game_loaded")
	#close()

func confirm_save(answer,save_path,SlotNode):
	if not answer:
		return
	Audio.play_sfx("ui_accept")
	#var start_index = get_item_parent().get_child_count() * (sys_data.data.current_page - 1)
	#slot_index = start_index + slot_index
	#add thumbnail
	var Subview = get_tree().get_nodes_in_group("Subview")
	if Subview.size() > 0:
		Subview = Subview[0] as Viewport
		var ThumbData = Subview.get_texture().get_data()
		ThumbData.flip_y()
		ThumbData.shrink_x2()
		ThumbData.shrink_x2()
		#ThumbData.convert(Image.FORMAT_ETC)
		ThumbData.compress(Image.COMPRESS_S3TC,Image.COMPRESS_SOURCE_SRGB,0.7)
		loaded_data.data.thumb = ThumbData
		#print("Thumbnail created!")
	#add fate_time
	loaded_data.data.datetime = OS.get_datetime()
	#read file
	#var save_path:String = Func.get_file_path("data") %slot_index
	var save_pepper:String = Func.get_file_pepper("data")
	#print("Save data path -> %s, pepper -> %s" %[save_path,save_pepper])
	var SaveFile:File = File.new()
	var SaveDir:Directory = Directory.new()
	if not SaveDir.dir_exists(save_path.get_base_dir()):
		SaveDir.make_dir_recursive(save_path.get_base_dir())
	SaveFile.open_encrypted_with_pass(save_path,File.WRITE,save_pepper)
	#print(var2str(loaded_data))
	SaveFile.store_var(loaded_data,true)
	SaveFile.close()
	change_last_page(last_page)
	change_last_slot(last_slot)
	update_slot_item(SlotNode,loaded_data.data)

func get_data():
	return loaded_data

func set_data(value = null):
	loaded_data = value

func load_data(value):
	var value_choice = value.choice
	var value_id = value.id
	loaded_data.data.choice = value_choice
	loaded_data.data.dialog = value_id
	loaded_data.data.chapter = -1

func get_quick_data():
	return quick_data

func save_game_quick():
	#var NewQuickData = DictionaryData.new()
	#print("Add data to quick save!")
	quick_data = loaded_data.duplicate()

func load_game_quick():
	loaded_data = quick_data.duplicate()

func set_confirmation(value):
	confirmation = value

func is_readed(id):
	return false if get_readed_range(id) < 0 else true

func get_readed_range(id):
	for index in sys_data.data.readed_line.size():
		if Func.is_inside_range(id,sys_data.data.readed_line[index]):
			return index
	return -1

func add_readed(id):
	#print("")
	if is_readed(id):
		#print("  > %s already readed!" %id)
		return
	#print("  > adding %s" %id)
	var range_group_prev = null
	var range_group_next = null
	for index in sys_data.data.readed_line.size():
		if sys_data.data.readed_line[index].y == float(id - 1):
			range_group_prev = index
		if sys_data.data.readed_line[index].x == float(id + 1):
			range_group_next = index
	if range_group_prev == null and range_group_next == null:
		sys_data.data.readed_line.append(Vector2(id,id))
	else:
		if range_group_prev == null:
			sys_data.data.readed_line[range_group_next].x = float(id)
		else:
			sys_data.data.readed_line[range_group_prev].y = float(id)
			if range_group_next != null:
				sys_data.data.readed_line[range_group_prev].y = sys_data.data.readed_line[range_group_next].y
				sys_data.data.readed_line.remove(range_group_next)
	save_sys_data()
