extends Node2D

const CPS = 30

enum AdvanceMode {
	Normal,
	Auto,
	Skip
}

export(Array) var ModeNodepaths

onready var NormalButton = $UILayer/UI/List/BottomPanel/List/DialogPanel/List/Controls/Margin/Autos/Normal
onready var QuickLoad = $UILayer/UI/List/BottomPanel/List/DialogPanel/List/Controls/Menu/QLoad
onready var NameNode = $UILayer/UI/List/BottomPanel/List/DialogPanel/List/NameMargin/Name
onready var NameDecor = $UILayer/UI/List/BottomPanel/List/DialogPanel/List/NameMargin/Name/Decor
onready var NameTweener = $UILayer/UI/List/BottomPanel/List/DialogPanel/List/NameMargin/NameTweener
onready var DialogNode = $UILayer/UI/List/BottomPanel/List/DialogPanel/List/DialogMargin/Dialog
onready var DialogTweener = $UILayer/UI/List/BottomPanel/List/DialogPanel/List/DialogMargin/DialogTweener
onready var NexterTree = $UILayer/UI/List/BottomPanel/Nexter/Content/Central/AnimationTree
onready var BlackScreen = $UILayer/UI/BlackScreen
onready var ChapterChange = $UILayer/UI/BlackScreen
onready var BGContainer = $Viewport/VN/BG
onready var CharaContainer = $Viewport/VN/Chara
onready var CGContainer = $Viewport/VN/CG
onready var VFXContainer = $Viewport/VN/VFX
onready var DialogLog = $UILayer/UI/Log
onready var AdvanceButton = $UILayer/UI/Advance
onready var VoicePlayer = $Voice
onready var Choice = $UILayer/UI/Choice
onready var DialogTimer = $DialogTimer
onready var AutoTimer = $AutoTimer
onready var SkipTimer = $SkipTimer

var ModeNodes = []
var Backgrounds = []
var Musics = []
var Characters = []
var CGs = []
var Portraits = {}
var SaveData:DictionaryData = null
var SysData:DictionaryData = null
var QuickData:DictionaryData = null

var PlayRoute = Route.new()
var ChaptersData = Chapters.new()

var current_mode = AdvanceMode.Normal
var advancing = false
var first = true
var c_logdata = {}

signal advanced
signal advanced_extra
signal on_credit

func _ready():
	init_mode_nodes()
	init_backgrounds()
	init_musics()
	init_characters()
	init_cgs()
	update_mode_nodes()
	update_quick_load()
	init_gameplay()
	yield(get_tree(),"idle_frame")
	advance(1.0,0)
	set_process_input(false)

func _inpute(event):
	if event is InputEventMouseButton:
		if event.button_index == 1:
			set_normal_mode()
	if event is InputEventScreenTouch:
		if event.pressed:
			set_normal_mode()
	if event is InputEventJoypadButton:
		if event.pressed:
			set_normal_mode()

func set_mode(index):
	Audio.play_sfx("ui_accept")
	print("ccc Current index %s, this index %s" %[current_mode,index])
	if current_mode == index:
		if current_mode == AdvanceMode.Skip:
			set_process_input(false)
		current_mode = 0
		NormalButton.pressed = true
	else:
		current_mode = index
		if current_mode != AdvanceMode.Normal:
			advance()
		if current_mode == AdvanceMode.Skip:
			set_process_input(true)
		#print("Current mode is -> ")
	update_mode_nodes()

func init_mode_nodes():
	for path in ModeNodepaths:
		ModeNodes.append(get_node(path))

func init_backgrounds():
	var BGDir:Directory = Directory.new()
	var bg_dir = "res://Data/BG/"
	if BGDir.open(bg_dir) == OK:
		BGDir.list_dir_begin(true)
		var file_name = BGDir.get_next()
		while file_name != "":
			var bg_path = bg_dir.plus_file(file_name)
			var BGData = load(bg_path)
			Backgrounds.append(BGData)
			file_name = BGDir.get_next()

func init_musics():
	var BGMDir:Directory = Directory.new()
	var bgm_dir = "res://Data/BGM/"
	if BGMDir.open(bgm_dir) == OK:
		BGMDir.list_dir_begin(true)
		var file_name = BGMDir.get_next()
		while file_name != "":
			var bgm_path = bgm_dir.plus_file(file_name)
			var BGMData = load(bgm_path)
			Musics.append(BGMData)
			file_name = BGMDir.get_next()

func init_characters():
	var CharaDir:Directory = Directory.new()
	var chara_dir = "res://Data/Chara/"
	if CharaDir.open(chara_dir) == OK:
		CharaDir.list_dir_begin(true)
		var file_name = CharaDir.get_next()
		while file_name != "":
			var chara_path = chara_dir.plus_file(file_name)
			var CharaData = load(chara_path)
			Characters.append(CharaData)
			file_name = CharaDir.get_next()

func init_cgs():
	var CGDir:Directory = Directory.new()
	var cg_dir = "res://Data/CG/"
	if CGDir.open(cg_dir) == OK:
		CGDir.list_dir_begin(true)
		var file_name = CGDir.get_next()
		while file_name != "":
			var cg_path = cg_dir.plus_file(file_name)
			var CGData = load(cg_path)
			CGs.append(CGData)
			file_name = CGDir.get_next()

func init_gameplay():
	SaveData = VNData.get_data()
	PlayRoute.connect("ended",self,"emit_signal",["on_credit"])
	
	for CGItem in CGContainer.get_children():
		CGItem.free()
	
	for BGItem in BGContainer.get_children():
		BGItem.free()
	
	if SaveData == null:
		print("Save data is null")
		SaveData = DictionaryData.new()
		SaveData.data.choice = {}
		SaveData.data.thumb = null
		
		PlayRoute.advance()
		SaveData.data.dialog = int(PlayRoute.get_current_node_value().x)
		SaveData.data.chapter = ChaptersData.get_chapter_id(SaveData.data.dialog)
		SaveData.data.datetime = {}
		VNData.set_data(SaveData)
	else:
		if typeof(SaveData.data.choice) == TYPE_ARRAY:
			SaveData.data.choice = {}
		if SaveData.data.chapter < 0:
			SaveData.data.chapter = ChaptersData.get_chapter_id(SaveData.data.dialog)
		print("Loaded data -> dialog = %s, datetime = %s, choice = %s" %[SaveData.data.dialog,SaveData.data.datetime,SaveData.data.choice])
		PlayRoute.resume(SaveData.data.dialog,SaveData.data.choice)
	print("Save data -> %s" %SaveData.data)
	init_previous_effects()
	init_dialog_log()

func init_previous_effects():
	var lookup_range = PlayRoute.get_dialog_ids_inside_range(ChaptersData.get_chapter_range(SaveData.data.chapter),SaveData.data.dialog)
	var pref_effects = ChaptersData.get_prev_effects(SaveData.data.dialog,lookup_range)
	
	#displaying effects
	for key in pref_effects.keys():
		if key == "portrait":
			print("> prev effect %s" %var2str(pref_effects[key]))
			for p_id in pref_effects[key]:
				call("set_portrait", pref_effects[key][p_id])
		else:
			#print("Calling method set_%s" %key)
			var method_name = "set_%s" %key
			if has_method(method_name):
				call(method_name,pref_effects[key])

func init_dialog_log():
	var log_array = PlayRoute.create_log(SaveData.data.dialog,SaveData.data.choice.duplicate(true),Func.get_dialog_log_max_count())
	log_array = ChaptersData.add_log_name(log_array.duplicate())
	
	for d_log in log_array:
		if d_log.has("a_id"):
			var a_id = d_log["a_id"]
			if d_log["name"] == "":
				d_log["name"] = Characters[a_id].data["name"]
			d_log["color"] = Color.transparent
			if Characters[a_id].data.name != null:
				d_log["color"] = Characters[a_id].data.color
	
	log_array.resize(Func.get_dialog_log_max_count())
	#get_tree().call_group("LogItem","set_current",false)
	DialogLog.init_dialog_log(log_array)

func update_mode_nodes():
	ModeNodes[0].visible = false if current_mode == AdvanceMode.Normal else true
	if current_mode != AdvanceMode.Normal:
		for i in range(1,ModeNodes.size()):
			ModeNodes[i].visible = true if i == current_mode else false

func qsl_pressed(mode):
	var conf_text = "conf_qsave" if mode == VNData.Mode.Save else "conf_qload"
	Confirmation.connect("answered",self,"qsl_answer",[mode],CONNECT_ONESHOT)
	Confirmation.open(conf_text)

func qsl_answer(ans,mode):
	#Confirmation.disconnect("answered",self,"qsl_answer")
	if ans:
		if mode == VNData.Mode.Save:
			quick_save()
		elif mode == VNData.Mode.Load:
			quick_load()
	AdvanceButton.grab_focus()

func advance(start_delay = 0, addition = 1):
	AutoTimer.stop()
	DialogTimer.stop()
	SkipTimer.stop()
	#print("Advance!! [advancing = %s]" %advancing)
	if advancing:
		return
	if DialogNode.percent_visible == 1:
		print(" > Go next dialog")
		advancing = true
		
		#initialize_node(addition)
		#yield(initialize_node(addition),"completed")
		var init_node = initialize_node(addition)
		var paused = false
		
		if init_node is GDScriptFunctionState:
			paused = true
			init_node = yield(init_node,"completed")
		
		var curr_d_id = float(SaveData.data.dialog)
		if not VNData.is_readed(curr_d_id):
			VNData.add_readed(curr_d_id)
			if current_mode == AdvanceMode.Skip:
				set_normal_mode()
		
		#print("-- RUN EFFECT SEARCH --")
		initialize_dialog()
		
		#initialize_effects()
		#yield(initialize_effects(),"completed")
		var init_efx = initialize_effects()
		if init_efx is GDScriptFunctionState:
			paused = true
			init_efx = yield(init_efx,"completed")
		
		#print("Before yield extra")
		if paused:
			yield(self,"advanced_extra") 
		#print("After yield extra")
		
		if start_delay > 0 and not paused:
			yield(get_tree().create_timer(start_delay),"timeout")
		
		typewrite()
		
		if first:
			first = false
	else:
		print(" > Skip Type write")
		DialogTweener.stop_all()
		NexterTree.set("parameters/Transition/current",1)
		set_write_progress(1.0)

func finish_dialog():
	print("Enter finish dialog, current mode -> %s" %current_mode)
	if current_mode == AdvanceMode.Auto:
		AutoTimer.paused = false
		AutoTimer.start()

func set_write_progress(progress):
	DialogNode.percent_visible = progress
	for i in DialogNode.Glows.size():
		DialogNode.Glows[i].percent_visible = progress

func initialize_node(addition):
	if addition > 0:
		var next_d_id = SaveData.data.dialog + addition
		var dnode_range = PlayRoute.get_current_node_value()
		
		if Func.is_inside_range(next_d_id,dnode_range):
			#still in this node
			SaveData.data.dialog = next_d_id
			if not Func.is_inside_range(float(next_d_id),ChaptersData.get_chapter_range(SaveData.data.chapter)):
				SaveData.data.chapter = ChaptersData.get_chapter_id(next_d_id)
				if SaveData.data.chapter != -1:
					if not first:
						set_bgm(0)
						ChapterChange.start(current_mode != AdvanceMode.Normal)
						yield(self,"advanced")
			advancing = false
		else:
			#advance to next node
			PlayRoute.connect("advanced",self,"start_dialog",[],CONNECT_ONESHOT)
			PlayRoute.connect("choice_needed",Choice,"choice_open",[],CONNECT_ONESHOT)
			PlayRoute.advance()
			if PlayRoute.get_current_node_tyoe() == PlayRoute.NodeType.Choice:
				if not Choice.is_connected("selected",self,"select_choice"):
					Choice.connect("selected",self,"select_choice",[PlayRoute.current_node],CONNECT_ONESHOT)
			
			print("-- before yield in initialize_node function --")
			yield(self,"advanced")
			print("-- after yield in initialize_node function --")
			
			if PlayRoute.is_connected("choice_needed",Choice,"choice_open"):
				PlayRoute.disconnect("choice_needed",Choice,"choice_open")
	else:
		advancing = false

func select_choice(selected,choice_node):
	PlayRoute.select_choice(selected)
	SaveData.data.choice[str(choice_node)] = selected

func start_dialog():
	print("-- function start dialog --")
	#SaveData.data.dialog = int(PlayRoute.get_current_node_value().x) - 1
	SaveData.data.dialog = int(PlayRoute.get_current_node_value().x)
	advancing = false
	#emit_signal("advanced")
	#emit_signal("advanced_extra")
	call_deferred("emit_signal","advanced")
	call_deferred("emit_signal","advanced_extra")

func initialize_dialog():
	c_logdata = {}
	var dialog_id = SaveData.data.dialog
	var char_id = ChaptersData.get_char_id(SaveData.data.chapter,SaveData.data.dialog)
	var char_name = Characters[char_id].data.name
	#print("Char id -> %s, name -> %s" %[char_id,char_name])
	var prev_col = NameNode.modulate
	var next_col
	get_tree().call_group("Character","set_active",false)
	if char_name == null:
		next_col = Color.transparent
		#NameNode.modulate = Color.transparent
	else:
		#NameNode.modulate = Color.white
		c_logdata["name"] = char_name
		
		next_col = Characters[char_id].data.color
		#NameNode.modulate = Characters[char_id].data.color
		NameNode.set_text(char_name)
		if Portraits.has(char_id):
			#Portraits[char_id].set_active(true)
			Portraits[char_id].call_deferred("set_active",true)
	c_logdata["dialog"] = "dialog_%s" %dialog_id
	c_logdata["id"] = int(dialog_id)
	c_logdata["choice"] = SaveData.data.choice.duplicate(true)
	c_logdata["color"] = next_col
	#Need to code for skipping
	NameTweener.stop_all()
	NameTweener.interpolate_property(NameNode,"modulate",prev_col,next_col,0.3,Tween.TRANS_LINEAR,Tween.EASE_IN)
	NameTweener.start()
	var dialog_code = "dialog_%s" %SaveData.data.dialog
	DialogNode.set_text(dialog_code)
	set_write_progress(0)

func initialize_effects():
	var effects = ChaptersData.get_effects(SaveData.data.chapter,SaveData.data.dialog)
	#print("Effect for chapter %s dialog %s is %s" %[SaveData.data.chapter,SaveData.data.dialog,var2str(effects)])
	
	#var black_screen = false
	if effects.has("screen") and not first and current_mode != AdvanceMode.Skip:
		if typeof(effects["screen"]) == TYPE_INT and effects["screen"] == 0:
			#black_screen = true
			set_bgm(0)
			BlackScreen.start(current_mode == AdvanceMode.Auto)
			yield(self,"advanced")
			
			if effects.has("bg"):
				get_tree().call_group("VNBackground","free")
	#if black_screen and effects.has("bg"):
	#	get_tree().call_group("VNBackground","free")
	
	for effect in effects:
		var method_name = "set_%s" %effect
		if method_name == "set_name":
			method_name = "set_actor_name"
			c_logdata["name"] = effects[effect]
		var method_value = effects[effect]
		#print("Init calling %s" %method_name)
		if has_method(method_name):
			if effect == "portrait" or effect == "vfx":
				#var convert_val = convert_portrait_dic(method_value)
				#print("Found effect %s -> %s" %[effect,var2str(method_value)])
				var convert_val = ChaptersData.convert_effect_dic(method_value, false if effect == "portrait" else true)
				#print("Converted method value -> %s" %var2str(convert_val))
				for p_id in convert_val.keys():
					call(method_name,convert_val[p_id])
			#elif effect == "vfx":
			#	for p_id in method_value.keys():
			#		call(method_name,method_value[p_id])
			else:
				call(method_name,method_value)
	#DialogLog.add_dialog(c_logdata)

func set_actor_name(value):
	NameNode.set_text(value)

func set_screen(value):
	if typeof(value) == TYPE_INT and value == 0:
		return
	
	var param
	if typeof(value) == TYPE_ARRAY:
		param = value[1]
		value = value[0]
	
	var method
	match value:
		1:
			method = "shake"
		2:
			method = "flash"
	
	if method != null and current_mode != AdvanceMode.Skip:
		if param == null:
			get_tree().call_group("Camera",method)
		else:
			get_tree().call_group("Camera",method,param)

func set_bgm(value):
	Audio.change_bgm(int(value) - 1)

func set_bg(value):
	get_tree().call_group("VNBackground","remove")
	#print("BG size is -> %s" %get_tree().get_nodes_in_group("VNBackground").size())
	#get_tree().call_group_flags(SceneTree.GROUP_CALL_REALTIME,"VNBackground","remove")
	var NewBG = Backgrounds[value].data.scene.instance()
	print("Set BG to %s" %NewBG.name)
	BGContainer.add_child(NewBG)
	BGContainer.move_child(NewBG,0)

func set_cg(value):
	if typeof(value) == TYPE_BOOL:
		get_tree().call_group("VNCG","advance" if value else "remove")
	else:
		get_tree().call_group("VNCG","remove")
		VNData.unlock_cg(int(value))
		var NewCG = CGs[int(value)].data.scene.instance()
		CGContainer.add_child(NewCG)
		CGContainer.move_child(NewCG,0)
		NewCG.enter()

# this method only accept array value, convert first readed data from DialogResource using convert_portrait_dic
func set_portrait(value):
	var portrait_index = value[0]
	var portrait_params = value[1]
	print(">> Set portrait index -> %s" %str(portrait_index))
	if not Portraits.has(portrait_index):
		var newChar = Characters[portrait_index].data.portrait.instance()
		CharaContainer.add_child(newChar)
		Portraits[portrait_index] = newChar
		print("Created new Portrait -> %s" %newChar.name)
		#visible = false
		Portraits[portrait_index].visible = false
	if not Portraits[portrait_index].visible:
		print("Portrait %s is not displayed yet. Perform enter!" %Portraits[portrait_index].name)
		Portraits[portrait_index].enter(current_mode == AdvanceMode.Skip)
	if portrait_params == null:
		Portraits[portrait_index].remove(current_mode == AdvanceMode.Skip)
	if typeof(portrait_params) == TYPE_DICTIONARY:
		for key in portrait_params.keys():
			if current_mode == AdvanceMode.Skip and key == "anim":
				continue
			Portraits[portrait_index].call("set_%s" %key,portrait_params[key])

func set_vfx(value):
	#print("Set vfx -> %s" %var2str(value))
	if typeof(value[0]) == TYPE_ARRAY:
		value = value[0]
	var vfx_name = value[0]
	var one_time = ChaptersData.is_vfx_onetime(vfx_name)
	
	if current_mode == AdvanceMode.Skip and one_time:
		return
	
	var res_path = "res://Objects/VN/Effects/%s.tscn" %vfx_name
	if ResourceLoader.exists(res_path):
		var vfx_param = value[1]
		var current_VFX = null
		if one_time:
			current_VFX = load(res_path).instance()
			current_VFX.name = vfx_name
			current_VFX.vfx_name = vfx_name
			VFXContainer.add_child(current_VFX)
		else:
			var Props = get_tree().get_nodes_in_group("Prop")
			for prop in Props:
				if prop.vfx_name == vfx_name:
					current_VFX = prop
					break
			if current_VFX == null:
				print("Create new Props!!")
				current_VFX = load(res_path).instance()
				current_VFX.name = vfx_name
				current_VFX.vfx_name = vfx_name
				VFXContainer.add_child(current_VFX)
				current_VFX.enter()
			#print("vfx_param -> %s" %vfx_param)
		if typeof(vfx_param) == TYPE_DICTIONARY:
			for key in vfx_param.keys():
				if key == "value":
					continue
				var param = vfx_param[key]
				if vfx_param.has("value"):
					var f_name = param
					param = []
					param.append(f_name)
					param.append(vfx_param["value"])
				current_VFX.call("set_%s" %key,param)
		

func typewrite():
	print("Typewriting -> %s" %tr(DialogNode.text))
	var voice_res = Func.get_voice(SaveData.data.chapter, SaveData.data.dialog)
	var duration = get_write_speed(tr(DialogNode.text),voice_res)
	
	if voice_res != null:
		#voice_res = voice_res as AudioStream
		#duration = voice_res.get_length() / 2
		VoicePlayer.stream = voice_res
		c_logdata["voice"] = voice_res
		VoicePlayer.play()
	else:
		VoicePlayer.stop()
		DialogTimer.paused = false
		DialogTimer.wait_time = duration
		DialogTimer.start()
		print("Starting dialog timer -> %s" %duration)
		var conn_list = DialogTimer.get_signal_connection_list("timeout")
		#print("Connection list -> %s" %conn_list.size())
		if conn_list.size() > 0:
			print("Dialog Timer connection -> %s" %var2str(conn_list[0].method))
	
	#applying speed
	#maybe move speed param to settings??
	var speed = 4
	duration /= speed
	
	DialogLog.add_dialog(c_logdata)
	
	#set_write_progress(0)
	if current_mode != AdvanceMode.Skip:
		NexterTree.set("parameters/Transition/current",0)
		DialogTweener.interpolate_method(self,"set_write_progress",0.01,1.0,duration,Tween.TRANS_LINEAR,Tween.EASE_IN)
		DialogTweener.interpolate_callback(NexterTree,duration,"set","parameters/Transition/current",1)
		DialogTweener.start()
	else:
		set_write_progress(1)
		SkipTimer.start()

func reload(delay = 0.1):
	VoicePlayer.stop()
	set_normal_mode()
	Choice.close()
	yield(get_tree().create_timer(delay),"timeout")
	print("Reloading!!!!")
	VNData.call_deferred("close")
	advancing = true
	BlackScreen.start()
	yield(BlackScreen,"start_ended")
	print("After yield black screen")
	#get_tree().call_group("Character","remove",true)
	get_tree().call_group_flags(SceneTree.GROUP_CALL_REALTIME,"Character","remove",true)
	get_tree().call_group_flags(SceneTree.GROUP_CALL_REALTIME,"Prop","queue_free")
	#yield(init_gameplay(),"completed")
	PlayRoute = Route.new()
	init_gameplay()
	yield(get_tree().create_timer(0.5),"timeout")
	print("After yield init gameplay")
	BlackScreen.end()
	yield(get_tree(),"idle_frame")
	print("After yield idle frame")
	advancing = false
	advance(1.0,0)
	var efk1_532 = ChaptersData.get_effects(1,532)
	print("Effect c1, d532 = %s" %var2str(efk1_532))

func set_normal_mode():
	var PressedButton = NormalButton.group.get_pressed_button()
	if PressedButton != NormalButton:
		#if current_mode == AdvanceMode.Skip:
		#	set_process_input(false)
		set_mode(0)
		NormalButton.pressed = true
		PressedButton.pressed = false

func quick_save():
	VNData.save_game_quick()
	update_quick_load()

func quick_load():
	VNData.load_game_quick()
	reload(0.3)

func update_quick_load():
	var quick_data = VNData.get_quick_data()
	QuickLoad.focus_mode = Control.FOCUS_NONE if quick_data == null else Control.FOCUS_ALL
	QuickLoad.mouse_filter = Control.MOUSE_FILTER_IGNORE if quick_data == null else Control.MOUSE_FILTER_STOP
	QuickLoad.disabled = true if quick_data == null else false
	QuickLoad.modulate = ColorN("white",0) if quick_data == null else ColorN("white",1)

func is_dialog_finished():
	if VoicePlayer.playing:
		return false
	return DialogNode.percent_visible == 1
	#return false

func get_write_speed(text:String,voice:AudioStream = null):
	if voice != null:
		return voice.get_length() / 2
	#var word_count = float(text.split(" ").size())
	#return word_count / float(WPS)
	return float(text.length()) / float(CPS)
