extends Node2D

class_name VNPortrait

onready var Portrait = get_child(0)
#onready var ExpressionTree:AnimationTree = $Expression
onready var ExpressionTree:AnimationTree = null if not has_node("Expression") else get_node("Expression")
onready var AnimNode:AnimationPlayer = null if not has_node("Anim") else get_node("Anim")
onready var CharTweener = $Tween
#onready var CharTweener = null if not has_node("Tween") else get_node("Tween")

var displayed = false

func enter(instant = false):
	print("%s portrait entered and %s" %[name,"instant" if instant else "not instant"])
	visible = true
	
	if instant:
		modulate = Color.white
		displayed = true
	else:
		#CharTweener.interpolate_property(self,"modulate",Color.white,Color.transparent,0.5,Tween.TRANS_LINEAR,Tween.EASE_IN)
		CharTweener.interpolate_property(self,"modulate",Color.transparent,Color.white,0.5,Tween.TRANS_LINEAR,Tween.EASE_IN)
		CharTweener.interpolate_property(self,"displayed",false,true,0.1,Tween.TRANS_LINEAR,Tween.EASE_IN)
		CharTweener.start()

func set_active(state):
	#print("Set active state of %s to %s" %[name,state])
	z_index = 1 if state else 0

func remove(instant = false):
	#CharTweener.interpolate_property(self,"modulate",Color.transparent,Color.white,0.5,Tween.TRANS_LINEAR,Tween.EASE_IN)
	displayed = false
	if instant:
		modulate = Color.transparent
		set_emot(0)
		set_col(Color.white.to_html())
		set_pos(0)
		set_size(1)
		visible = false
		print(">> End remove instant")
	else:
		CharTweener.interpolate_property(self,"modulate",Color.white,Color.transparent,0.5,Tween.TRANS_LINEAR,Tween.EASE_IN)
		CharTweener.interpolate_callback(self,0.5,"set_visible",false)
		CharTweener.interpolate_callback(self,0.5,"set_emot",0)
		CharTweener.interpolate_callback(self,0.5,"set_col",Color.white.to_html())
		CharTweener.interpolate_callback(self,0.5,"set_pos",0)
		CharTweener.interpolate_callback(self,0.5,"set_size",1)
		CharTweener.start()

func set_emot(value):
	if ExpressionTree != null:
		ExpressionTree.set("parameters/Transition/current",int(value))

func set_col(value):
	var target_color = Color(value)
	if displayed:
		CharTweener.interpolate_property(Portrait,"modulate",Portrait.modulate,target_color,0.5,Tween.TRANS_LINEAR,Tween.EASE_IN)
		CharTweener.start()
	else:
		Portrait.modulate = target_color

func set_pos(value):
	var vec2pos = Vector2.ZERO
	if value != 0:
		#vec2pos.x = OS.get_screen_size().x * 0.5 * value
		vec2pos.x = Options.adapted_size.x * 0.5 * value
	if displayed:
		#print("%s portrait is tweened pos" %name)
		CharTweener.interpolate_property(self,"position",position,vec2pos,0.5,Tween.TRANS_LINEAR,Tween.EASE_IN)
		CharTweener.start()
	else:
		#print("%s portrait is instant pos" %name)
		position = vec2pos

func set_size(value):
	var vec2scale = Vector2(value,value)
	if displayed:
		CharTweener.interpolate_property(self,"scale",scale,vec2scale,0.5,Tween.TRANS_LINEAR,Tween.EASE_IN)
		CharTweener.start()
	else:
		scale = vec2scale

func set_effect(value):
	var effect_dic = value as Dictionary
	for key in effect_dic.keys():
		pass

func set_anim(value):
	if AnimNode != null:
		AnimNode.play(value)
