tool
extends PanelContainer

const ItemIns = preload("res://addons/dialog_editor/LineItem.tscn")
const LangAddItem = preload("res://addons/dialog_editor/LangAddItem.tscn")
const EffectItem = preload("res://addons/dialog_editor/EffectItem.tscn")

onready var LangOption = $List/TopBar/VList/List/LangOption
onready var ChoiceLabel = $List/BottomPanel/Right/Choice/List/Number
onready var LineLabel = $List/BottomPanel/Right/Line/List/Number
onready var ItemParent = $List/ItemPanel/Dialogs/List
onready var ChooseDataDialog = $DataChoose
onready var TransFilesDialog = $TransChoose
onready var ChapterAddDialog = $ChapterAdd
onready var FileLabel = $List/TopBar/VList/List/DataPath
onready var ChapterList = $ChapterAdd/List/Panel/List/Chapters
onready var LangList = $ChapterAdd/List/Langs
onready var ChapterChanger = $List/TopBar/VList/List2/Chapter
onready var ChapterTitle = $List/TopBar/VList/List2/Title
onready var EffectAddDialog = $EffectAdd
onready var EffectOptions = $EffectOptions
onready var EffectList = $EffectOptions/Main/List
onready var BGItemList = $EffectOptions/Main/List/BG
onready var BGMItemList = $EffectOptions/Main/List/BGM
onready var CGItemList = $EffectOptions/Main/List/CG/List
onready var VFXItemList = $EffectOptions/Main/List/VFX/List
onready var PortraitItemList = $EffectOptions/Main/List/Portrait/List
onready var ScreenItemList = $EffectOptions/Main/List/Screen/List
onready var ScreenItemParamContainer = $EffectOptions/Main/List/Screen/Parameters
onready var RMBMenu = $RMBMenu

var Config:ConfigFile = null
var ChapterData:DictionaryData = null
var DialogData:DictionaryData = null
var Translations = []
var Languages = []
var NameLangs = []
var Characters = []
var EffectTrigger = null

var config_path = "res://addons/dialog_editor/dialog_editor_config.cfg"
var data_setting = "vn/gameplay/data/chapters"
var current_lang = -1
var current_chapter = -1
var chapter_select = -1
var chapter_select_lang = -1
var rmb_selected = -1

onready var EffectsList = {
	"bg" : BGItemList,
	"bgm" : BGMItemList,
	"cg" : CGItemList,
	"portrait" : PortraitItemList,
	"screen" : ScreenItemList,
	"vfx" : VFXItemList
}

var PortraitParams = {}
var ScreenParam = {}

# Called when the node enters the scene tree for the first time.
func _ready():
	#init_config()
	init_langs()
	init_data()
	init_charas()
	init_backgrounds()
	init_bgm()
	init_cg()
	init_portrait()
	init_screen_effects()
	init_vfx()
	#ChapterChanger.value = current_chapter
	update_chapter(current_chapter,true)
	#init_items()
	#update_items(false)

#func init_config():
#	Config = ConfigFile.new()
#	var err = Config.load(config_path)
#	if err == OK:
#		pass
#	else:
#		Config = null

func _input(event):
	if event is InputEventMouseButton:
		if event.pressed and event.button_index == BUTTON_RIGHT:
			#print("Mouse Pos %s" %event.global_position)
			var mouse_pos = event.global_position
			var current_focus = get_focus_owner()
			#print("Current index %s" %current_focus.get_index())
			#print("Current index parent %s" %current_focus.get_parent())
			if current_focus.get_parent() == ItemParent:
				rmb_selected = current_focus.get_index()
			else:
				rmb_selected = -1
			RMBMenu.rect_global_position = mouse_pos
			RMBMenu.popup()

func exec_rmb_func(index):
	#print("> RMB Menu %s executed" %index)
	match index:
		0:
			if rmb_selected >= 0:
				var chapter_range = ChapterData.data[current_chapter]["range"]
				var line_count = chapter_range.y - chapter_range.x
				#print("> Current line index is %s" %rmb_selected)
				#print("> Max line index is %s" %line_count)
				var affected = 0
				
				if get_dialog_data().data.has(rmb_selected):
					get_dialog_data().data.erase(rmb_selected)
				
				for i in range(rmb_selected + 1, line_count + 1):
					if get_dialog_data().data.has(i):
						var current_value = get_dialog_data().data[i].duplicate(true)
						get_dialog_data().data[i-1] = current_value
						get_dialog_data().data.erase(i)
						affected += 1
				
				save_dialog_data()
				if affected > 0:
					update_chapter(current_chapter,true)

func init_langs():
	var LangDir:Directory = Directory.new()
	var TranlationList = ProjectSettings.get_setting("locale/translations")
	var lang_dir = "res://Data/Langs/"
	var lang_id = 0
	LangOption.clear()
	for i in LangList.get_child_count():
		var LangCur = LangList.get_child(i).queue_free()
	if LangDir.open(lang_dir) == OK:
		LangDir.list_dir_begin(true)
		var file_name = LangDir.get_next()
		while file_name != "":
			var lang_path = lang_dir.plus_file(file_name)
			var LangData = load(lang_path)
			Languages.append(LangData.data.id)
			LangOption.add_item(LangData.data.name)
			file_name = LangDir.get_next()
			
			var CNameLang = null
			for transpath in TranlationList:
				if transpath.find("VNTexts") >= 0:
					if transpath.find(LangData.data.id) >= 0:
						CNameLang = load(transpath)
						NameLangs.append(CNameLang)
						break
			
			var NewLangAdd = LangAddItem.instance()
			NewLangAdd.name = "Lang%s" %lang_id
			LangList.add_child(NewLangAdd)
			NewLangAdd.get_node("List/Code").text = "%s:" %LangData.data.id
			NewLangAdd.get_node("List/Add").connect("pressed",self,"open_add_chapter_trans",[lang_id])
			
			lang_id += 1
	if not Languages.empty():
		current_lang = get_config().get_value("settings","current_lang",0)
		LangOption.select(current_lang)
		update_lang(current_lang)

func init_charas():
	var CharaDir:Directory = Directory.new()
	var chara_dir = "res://Data/Chara/"
	if CharaDir.open(chara_dir) == OK:
		CharaDir.list_dir_begin(true)
		var file_name = CharaDir.get_next()
		while file_name != "":
			var chara_path = chara_dir.plus_file(file_name)
			var CharaData = load(chara_path)
			Characters.append(CharaData)
			file_name = CharaDir.get_next()

func init_backgrounds():
	var BGDir:Directory = Directory.new()
	var bg_dir = "res://Data/BG/"
	BGItemList.clear()
	#var char_list = []
	if BGDir.open(bg_dir) == OK:
		BGDir.list_dir_begin(true)
		var file_name = BGDir.get_next()
		while file_name != "":
			var bg_path = bg_dir.plus_file(file_name)
			var BGData = load(bg_path)
			BGItemList.add_item(BGData.data.name)
			file_name = BGDir.get_next()

func init_bgm():
	var BGMDir:Directory = Directory.new()
	var bgm_dir = "res://Data/BGM/"
	BGMItemList.clear()
	#var char_list = []
	BGMItemList.add_item("[none]")
	if BGMDir.open(bgm_dir) == OK:
		BGMDir.list_dir_begin(true)
		var file_name = BGMDir.get_next()
		while file_name != "":
			var bgm_path = bgm_dir.plus_file(file_name)
			var BGMData = load(bgm_path)
			BGMItemList.add_item(BGMData.data.id)
			file_name = BGMDir.get_next()

func init_cg():
	var CGDir:Directory = Directory.new()
	var cg_dir = "res://Data/CG/"
	CGItemList.clear()
	#var char_list = []
	if CGDir.open(cg_dir) == OK:
		CGDir.list_dir_begin(true)
		var file_name = CGDir.get_next()
		while file_name != "":
			var cg_path = cg_dir.plus_file(file_name)
			var CGData = load(cg_path)
			CGItemList.add_item(CGData.data.title)
			file_name = CGDir.get_next()

func init_vfx():
	var VFXDir:Directory = Directory.new()
	var vfx_dir = "res://Objects/VN/Effects/"
	VFXItemList.clear()
	if VFXDir.open(vfx_dir) == OK:
		VFXDir.list_dir_begin(true)
		var file_name = VFXDir.get_next()
		while file_name != "":
			var vfx_path = vfx_dir.plus_file(file_name)
			var vfx_name = vfx_path.get_basename().get_file()
			VFXItemList.add_item(vfx_name)
			file_name = VFXDir.get_next()

func init_portrait():
	var names = get_translated_chara()
	PortraitItemList.clear()
	var ExpressionsParent = PortraitItemList.get_parent().get_node("Parameters/Expression/List")
	var AnimationsParent = PortraitItemList.get_parent().get_node("Parameters/Animation/List")
	for Child in ExpressionsParent.get_children():
		Child.free()
	for i in Characters.size():
		var sel_enable = false if Characters[i].data.portrait == null else true
		#print("Portrait %s (%s) is %s" %[i,names[i],str(sel_enable)])
		PortraitItemList.add_item(names[i])
		PortraitItemList.set_item_disabled(i,!sel_enable)
		var CharaNode = Characters[i].data.portrait
		if CharaNode != null:
			var CharaTemp = CharaNode.instance()
			
			# CREATE EXPRESSIONS LIST
			var ExpressionOp = OptionButton.new()
			ExpressionOp.name = str(i)
			ExpressionOp.visible = false
			
			if not CharaTemp.has_node("Expression"):
				ExpressionOp.add_item("-")
				ExpressionOp.disabled = true
			else:
				var ExpressionNode:AnimationTree = CharaTemp.get_node("Expression")
				var TransitionNode:AnimationNodeTransition = ExpressionNode.tree_root.get_node("Transition")
				#var Expressions = []
				for j in TransitionNode.input_count:
					var ex_text = TransitionNode.get_input_name(j)
					#Expressions.append(ex_text)
					ExpressionOp.add_item(ex_text,j)
			#CharaTemp.free()
			ExpressionsParent.add_child(ExpressionOp)
			
			# CREATE ANIMATIONS LIST
			var AnimationOp = OptionButton.new()
			AnimationOp.name = str(i)
			AnimationOp.visible = false
			if not CharaTemp.has_node("Anim"):
				AnimationOp.add_item("-")
				AnimationOp.disabled = true
			else:
				var AnimNode:AnimationPlayer = CharaTemp.get_node("Anim")
				var AnimList:PoolStringArray = AnimNode.get_animation_list()
				#var Expressions = []
				for j in AnimList.size():
					AnimationOp.add_item(AnimList[j],j)
			AnimationsParent.add_child(AnimationOp)
			
			CharaTemp.free()
		else:
			var EmptyOp = OptionButton.new()
			EmptyOp.add_item("-")
			EmptyOp.disabled = true
			EmptyOp.name = str(i)
			EmptyOp.visible = false
			ExpressionsParent.add_child(EmptyOp)
			AnimationsParent.add_child(EmptyOp.duplicate())
	#Create Dictionary to lookup portrait params when initializing effects
	ExpressionsParent.get_child(0).visible = true
	var ParamParent = PortraitItemList.get_parent().get_node("Parameters")
	for i in ParamParent.get_child_count():
		var CurrentParam = ParamParent.get_child(i)
		PortraitParams[CurrentParam.editor_description] = CurrentParam

func init_screen_effects():
	var param_val = ScreenItemList.editor_description
	for line in param_val.split("\n"):
		var line_val = line.split(":")
		ScreenParam[int(line_val[0])] = line_val[1].split(",")

func get_translated_chara() -> PoolStringArray:
	var trans_charas = PoolStringArray([])
	for i in Characters.size():
		var cur_name = Characters[i].data.name
		if cur_name == null:
			cur_name = ""
		trans_charas.append(cur_name)
	if not NameLangs.empty():
		if NameLangs.size() > current_lang:
			if NameLangs[current_lang] != null:
				for i in trans_charas.size():
					trans_charas[i] = NameLangs[current_lang].get_message(trans_charas[i])
	return trans_charas

func update_lang(index):
	if index < Languages.size():
		if index != current_lang:
			current_lang = index
			update_config("settings","current_lang",current_lang)
			# UPDATE DIALOG
			update_items()
			
			# UPDATE CHARACTER
			
			# UPDATE CHAPTER TITLE
			update_chapter_title(current_chapter)

func init_data():
	var data_file
	if ProjectSettings.has_setting(data_setting):
		data_file = ProjectSettings.get_setting(data_setting)
	if data_file != "" and data_file != null:
		ChooseDataDialog.current_file = data_file
		ChooseDataDialog.current_dir = data_file.get_base_dir()
		FileLabel.text = data_file
		ChapterData = load(data_file)
	if ChapterData != null:
		if ChapterData.data.empty():
			print("Chapter Data not found")
			return
		else:
			var default_chapter = ChapterData.data.keys()[0]
			var c_chapter = get_config().get_value("settings","current_chapter",default_chapter)
			update_config("settings","current_chapter",c_chapter)
			current_chapter = c_chapter
			#update_chapter(c_chapter)
			#ChapterChanger.max_value = float(ChapterData.data.size() - 1)

#Used whwn change chapter
func init_items():
	if ChapterData == null:
		return
	if ChapterData.data.empty():
		return
	if current_chapter < 0:
		return
	#var curre
	var current_range = ChapterData.data[current_chapter]["range"]
	var item_count = int(current_range.y - current_range.x) + 1
	var child_count = ItemParent.get_child_count()
	var chara_list = get_translated_chara()
	#print("Chara List -> %s" %chara_list)
	#print("Characters length -> %s" %Characters.size())
	#return
	var loop_count = int(max(float(item_count),float(child_count)))
	
	for i in range(loop_count):
		var item_name = "Item%s" %i
		if i < item_count:
			if not ItemParent.has_node(item_name):
				var NewItem = ItemIns.instance()
				NewItem.name = item_name
				var CharaOption:OptionButton = NewItem.get_node("List/Character")
				#for chara in chara_list:
				#	CharaOption.add_item(chara)
				for j in chara_list.size():
					CharaOption.add_item(chara_list[j],j)
				#CharaOption.add_items(chara_list)
				
				ItemParent.add_child(NewItem)
			else:
				var ItemNode = ItemParent.get_node_or_null(item_name)
				if ItemNode != null:
					pass
				if not ItemNode.is_visible():
					ItemNode.show()
		else:
			var ItemNode = ItemParent.get_node_or_null(item_name)
			if ItemNode != null:
				ItemNode.hide()

func get_config() -> ConfigFile:
	if Config == null:
		Config = ConfigFile.new()
		Config.load(config_path)
	return Config

func update_config(section,key,value):
	var Conf:ConfigFile = get_config()
	Conf.set_value(section,key,value)
	Conf.save(config_path)

#used when change lang
func update_items(text_only = true):
	if ChapterData == null:
		return
	if ChapterData.data.empty():
		return
	if current_chapter < 0:
		return
	
	var start_num = int(ChapterData.data[current_chapter]["range"].x)
	var lang_code = Languages[current_lang]
	#var TransDic = get_config().get_value("settings","translations",{})
	var CurrentTrans:PHashTranslation = get_chapter_translations()[current_lang]
	var charas = get_translated_chara()
	
	var effect_ids = []
	if not text_only:
		#init effect id list
		var EffectsPar = EffectAddDialog.get_node("List")
		effect_ids.resize(EffectsPar.get_child_count())
		for j in EffectsPar.get_child_count():
			effect_ids[j] = EffectsPar.get_child(j).editor_description
				
	#print("Item Count = %s" %ItemParent.get_child_count())
	for i in range(ItemParent.get_child_count()):
		var item_num = start_num + i
		var dialog_id = "dialog_%s" %str(item_num )
		var line = ""
		var IItem = ItemParent.get_child(i)
		
		line = CurrentTrans.get_message(dialog_id)
		
		#print("%s %s -> %s" %[i,dialog_id,line])
		
		if line != "":
			var LineItem = IItem.get_node("List/Line")
			LineItem.text = line
			var NumberLabel = IItem.get_node("List/Number")
			NumberLabel.text = str(item_num + 1)
			# Edit options
			var CharaOption:OptionButton = IItem.get_node("List/Character")
			#print("Chara option size -> %s" %CharaOption.get_item_count())
			for j in charas.size():
				CharaOption.set_item_text(j,charas[j])
			
			if not text_only:
				# initiate current character
				var DData:DictionaryData = get_dialog_data()
				var chara_id = 0
				if DData.data.has(i):
					if DData.data[i].has("actor"):
						chara_id = DData.data[i]["actor"]
				CharaOption.select(chara_id)
				var chara_color = Characters[chara_id].data.color
				if chara_color == null:
					chara_color = Color.white
				chara_color = chara_color.lightened(0.4)
				CharaOption.set("custom_colors/font_color", chara_color)
				CharaOption.set("custom_colors/font_color_hover", chara_color)
				CharaOption.set("custom_colors/font_color_pressed", chara_color)
				if not CharaOption.is_connected("item_selected",self,"item_select_actor"):
					CharaOption.connect("item_selected",self,"item_select_actor",[CharaOption])
				
				var EffectButton:Button = IItem.get_node("List/Effects")
				if not EffectButton.is_connected("pressed",self,"item_effect_pressed"):
					EffectButton.connect("pressed",self,"item_effect_pressed",[EffectButton])
				#ItemParent.add_child(NewItem)
				
				#initiate effects
				EffectTrigger = EffectButton
				var EffectsContainer = IItem.get_node("List/EffectList")
				if EffectsContainer.get_child_count() > 0:
					for Child in EffectsContainer.get_children():
						Child.free()
				if DData.data.has(i):
					if DData.data[i].has("effects"):
						for key in DData.data[i].effects.keys():
							var ef_id = effect_ids.find(key)
							var func_name = "effect_value_changed"
							if has_method("%s_value_changed" %key):
								func_name = "%s_value_changed" %key
							if ef_id >= 0:
								var c_val = DData.data[i].effects[key]
								if typeof(c_val) == TYPE_DICTIONARY:
									#print("Dupli keys -> %s" %str(c_val.keys()))
									#var id_keys = DData.data[i].effects[key].keys().sort()
									var id_keys = DData.data[i].effects[key].keys()
									id_keys.sort()
									for id_key in id_keys:
										var NewEffect = item_effect_added(ef_id)
										EffectTrigger = NewEffect
										#effect_value_changed(c_val[id_key])
										call(func_name,c_val[id_key])
										EffectTrigger = EffectButton
								else:
									var NewEffect = item_effect_added(ef_id)
									EffectTrigger = NewEffect
									#effect_value_changed(c_val)
									call(func_name,c_val)
							EffectTrigger = EffectButton
		else:
			IItem.hide()
		
		

func item_select_actor(actor,OptionNode):
	var dialog_index = OptionNode.get_parent().get_parent().get_index()
	
	# if actor == 0 (narrator), if current none, return otherwise remove the actor key
	var have_actor_key = false
	if get_dialog_data().data.has(dialog_index):
		if get_dialog_data().data[dialog_index].has("actor"):
			have_actor_key = true
	
	if actor == 0:
		if have_actor_key:
			get_dialog_data().data[dialog_index].erase("actor")
			if get_dialog_data().data[dialog_index].empty():
				get_dialog_data().data.erase(dialog_index)
	else:
		if not get_dialog_data().data.has(dialog_index):
			get_dialog_data().data[dialog_index] = {}
		get_dialog_data().data[dialog_index]["actor"] = actor
		
		var chara_color = Characters[actor].data.color
		if chara_color == null:
			chara_color = Color.white
		chara_color = chara_color.lightened(0.4)
		OptionNode.set("custom_colors/font_color", chara_color)
		OptionNode.set("custom_colors/font_color_hover", chara_color)
		OptionNode.set("custom_colors/font_color_pressed", chara_color)
	
	save_dialog_data()

func item_effect_pressed(effect_button:Control):
	EffectTrigger = effect_button
	EffectAddDialog.rect_global_position = effect_button.rect_global_position
	EffectAddDialog.popup()

func item_effect_added(effect_id:int):
	var NewEffect = EffectItem.instance()
	var EffectRef = EffectAddDialog.get_node("List").get_child(effect_id)
	var name_id = EffectRef.name
	var color_item = EffectRef.get("custom_colors/font_color_disabled")
	var string_data = {
		"name" : name_id,
		"code" : EffectRef.editor_description
	}
	NewEffect.name = name_id
	NewEffect.text = name_id
	NewEffect.editor_description = var2str(string_data)
	NewEffect.get_node("BG").color = color_item
	NewEffect.connect("pressed",self,"effect_item_pressed",[NewEffect])
	EffectTrigger.get_parent().get_node("EffectList").add_child(NewEffect)
	EffectAddDialog.hide()
	return NewEffect

func effect_item_pressed(effect_item:Control):
	EffectTrigger = effect_item
	var item_id = effect_item.get_parent().get_parent().get_parent().get_index()
	var str_data = str2var(effect_item.editor_description)
	for i in EffectList.get_child_count():
		var CurChild = EffectList.get_child(i)
		#print("Size control %s is %s" %[CurChild.name,CurChild.rect_size])
		CurChild.visible = true if CurChild.name == str_data.name else false
	EffectOptions.popup_centered()
	EffectOptions.rect_global_position = effect_item.rect_global_position
	yield(get_tree(),"idle_frame")
	yield(get_tree(),"idle_frame")
	EffectOptions.set_as_minsize()
	

func effect_value_changed(value,hint_override = null):
	EffectOptions.hide()
	var item_id = EffectTrigger.get_parent().get_parent().get_parent().get_index()
	var str_data = str2var(EffectTrigger.editor_description)
	var val_str = hint_override
	if val_str == null:
		val_str = str(value)
		if typeof(value) == TYPE_INT:
			#val_str = EffectList.get_node(str_data["name"]).get_item_text(value)
			val_str = EffectsList[str_data.code].get_item_text(value)
	# ID based on name, used as index on dic, used when multiple same code in 1 line
	var val_id = int(EffectTrigger.name)
	
	str_data["value"] = value
	EffectTrigger.editor_description = var2str(str_data)
	EffectTrigger.text = "%s: %s" %[str_data.name, str_data.value]
	EffectTrigger.hint_tooltip = "%s: %s" %[str_data.name,val_str]
	
	var update = false
	var dialog_data = get_dialog_data()
	if not dialog_data.data.has(item_id):
		dialog_data.data[item_id] = {}
	if not dialog_data.data[item_id].has("effects"):
		dialog_data.data[item_id]["effects"] = {}
	if dialog_data.data[item_id].effects.has(str_data.code):
		var saved_effect = dialog_data.data[item_id].effects[str_data.code]
		# Already have multiple same code
		if typeof(saved_effect) == TYPE_DICTIONARY:
			if dialog_data.data[item_id].effects[str_data.code].has(val_id):
				#Editing already exist
				var c_val = dialog_data.data[item_id].effects[str_data.code][val_id]
				if typeof(c_val) != typeof(value):
					dialog_data.data[item_id].effects[str_data.code][val_id] = value
					update = true
				elif c_val != value:
					dialog_data.data[item_id].effects[str_data.code][val_id] = value
					update = true
			else:
				#add new
				dialog_data.data[item_id].effects[str_data.code][val_id] = value
				update = true
		else:
			if val_id == 0:
				if typeof(saved_effect) != typeof(value):
					dialog_data.data[item_id].effects[str_data.code] = value
					update = true
				elif saved_effect != value:
					dialog_data.data[item_id].effects[str_data.code] = value
					update = true
			else:
				dialog_data.data[item_id].effects[str_data.code] = {}
				dialog_data.data[item_id].effects[str_data.code][0] = saved_effect
				dialog_data.data[item_id].effects[str_data.code][val_id] = value
				update = true
	else:
		dialog_data.data[item_id].effects[str_data.code] = value
		update = true
	if update:
		save_dialog_data()
	

func vfx_pos_changed(value):
	var CStr = VFXItemList.get_parent().get_node("Position/List/CStr")
	CStr.text = str(value)

func vfx_value_applied(add = null):
	var selected_vfx = null
	if VFXItemList.is_anything_selected():
		var VFXPos = VFXItemList.get_parent().get_node("Position/List/Value")
		selected_vfx = VFXItemList.get_selected_items()[0]
		var selected_name = VFXItemList.get_item_text(selected_vfx)
		var value = []
		value.append(selected_name)
		var param = {}
		if add == null:
			param["action"] = "advance"
		else:
			if add:
				param["pos"] = VFXPos.value
			else:
				param["action"] = "remove"
		value.append(param)
		vfx_value_changed(value)

func vfx_value_changed(value):
	var hint = ""
	var param_hint = ""
	for key in value[1].keys():
		param_hint += ", %s = " %key
		if key == "pos":
			if value[1][key] == 0:
				param_hint += "Center"
			else:
				var posLR = "Left"
				if value[1][key] > 0:
					posLR = "Right"
				param_hint += "%s %s" %[abs(value[1][key]),posLR]
		else:
			param_hint += str(value[1][key])
	hint = " %s%s" %[value[0],param_hint]
	effect_value_changed(value,hint)

func portrait_effect_changed(index):
	# selecting character in portrait list, should change the expression list
	var EmotParent = PortraitParams["emot"].get_node("List")
	var AnimationParent = PortraitParams["anim"].get_node("List")
	for i in EmotParent.get_child_count():
		var CurrentEmot = EmotParent.get_child(i)
		var CurrentAnim = AnimationParent.get_child(i)
		CurrentEmot.visible = false if i != index else true
		CurrentAnim.visible = false if i != index else true

func portrait_param_toggled(visibility,index):
	yield(get_tree(),"idle_frame")
	yield(get_tree(),"idle_frame")
	EffectOptions.set_as_minsize()

func portrait_value_applied(adding = true):
	var selected_portrait = null
	# format:
	# -> id
	#	-> only adding without param, all param default
	# -> [id,null]
	#	-> removing portrait
	# -> [id,{dic}]
	#	-> adding with params
	if PortraitItemList.is_anything_selected():
		selected_portrait = PortraitItemList.get_selected_items()[0]
		var selected_name = PortraitItemList.get_item_text(selected_portrait)
		var value = []
		#var hint = ""
		if adding:
			value.append(selected_portrait)
			var params = {}
			var ParamParent = PortraitItemList.get_parent().get_node("Parameters")
			for i in ParamParent.get_child_count():
				var CurrentNode = ParamParent.get_child(i)
				if CurrentNode.visible:
					match i:
						0:
							var EmotOption:OptionButton = CurrentNode.get_node("List").get_child(selected_portrait)
							params[CurrentNode.editor_description] = EmotOption.selected
						1:
							var AnimOption:OptionButton = CurrentNode.get_node("List").get_child(selected_portrait)
							params[CurrentNode.editor_description] = AnimOption.get_item_text(AnimOption.selected)
						2:
							var RangeNode:HSlider = CurrentNode.get_node("List/Value")
							params[CurrentNode.editor_description] = RangeNode.value
						3:
							var RangeNode:HSlider = CurrentNode.get_node("List/Value")
							params[CurrentNode.editor_description] = RangeNode.value
						4:
							var ColNode:ColorPickerButton = CurrentNode.get_node("List/Color")
							params[CurrentNode.editor_description] = ColNode.color.to_html()
			if params.empty():
				value = selected_portrait
			else:
				value = [selected_portrait,params]
		else:
			value = [selected_portrait,null]
			#hint = "Remove %s Portrait" %selected_name
		portrait_value_changed(value)
	# just for checking otherwise comment it out
	#EffectOptions.hide()

func portrait_value_changed(value):
	var hint = ""
	var param_hint = ""
	var portrait_id
	if typeof(value) == TYPE_INT:
		portrait_id = value
	else:
		portrait_id = value[0]
		if value.size() > 1:
			if value[1] == null:
				hint = "Remove "
			else:
				for key in value[1].keys():
					var val_str = str(value[1][key])
					if key == "emot":
						var OptionNode:OptionButton = PortraitParams[key].get_node("List").get_child(portrait_id)
						val_str = OptionNode.get_item_text(value[1][key])
					if key == "anim":
						var OptionNode:OptionButton = PortraitParams[key].get_node("List").get_child(portrait_id)
						val_str = value[1][key]
					if key == "pos":
						if value[1][key] == 0:
							val_str = "Center"
						else:
							var posLR = "Left"
							if value[1][key] > 0:
								posLR = "Right"
							val_str = "%s %s" %[abs(value[1][key]),posLR]
					#print("Key -> %s" %key)
					param_hint += ", %s: %s" %[PortraitParams[key].name,val_str]
	hint += "%s Portrait%s" %[PortraitItemList.get_item_text(portrait_id),param_hint]
	#print("Portrait option value = %s, hint = %s" %[str(value),hint])
	# just for checking otherwise remove the comment
	effect_value_changed(value,hint)

func portrait_pos_changed(value):
	var CStr = PortraitParams["pos"].get_node("List/CStr")
	CStr.text = str(value)

func portrait_size_changed(value):
	var CStr = PortraitParams["size"].get_node("List/CStr")
	CStr.text = str(value)

func screen_value_changed(value):
	if ScreenParam.keys().has(value):
		var CParam = Array(ScreenParam[value])
		var show_item = 0
		for i in ScreenItemParamContainer.get_child_count():
			var ItemI = ScreenItemParamContainer.get_child(i)
			if ItemI.name == "Apply":
				continue
			ItemI.visible = CParam.has(ItemI.name)
			if ItemI.visible:
				show_item += 1
		if show_item > 0:
			ScreenItemParamContainer.show()
		EffectOptions.set_as_minsize()
	else:
		effect_value_changed(value)

func screen_value_applied():
	var selected_screen_effect = null
	
	if ScreenItemList.is_anything_selected():
		selected_screen_effect = ScreenItemList.get_selected_items()[0]
		var value = []
		if ScreenParam.keys().has(selected_screen_effect):
			var params = []
			for ival in ScreenParam[selected_screen_effect]:
				if ScreenItemParamContainer.has_node(ival):
					var ValNode = ScreenItemParamContainer.get_node(ival)
					if ValNode.visible:
						match ValNode.name:
							"Color":
								var ColorSelect:ColorPickerButton = ValNode.get_node("List/Color")
								params.append(ColorSelect.color)
			if not params.empty():
				value = [selected_screen_effect,params]
				effect_value_changed(value)

func remove_effect_pressed():
	var item_id = EffectTrigger.get_parent().get_parent().get_parent().get_index()
	var effect_data = str2var(EffectTrigger.editor_description)
	print("Remove id %s of effect %s (%s)" %[item_id,effect_data.name,effect_data.code])
	if effect_data.has("value"):
		var dialog_data = get_dialog_data()
		if dialog_data.data.has(item_id):
			if dialog_data.data[item_id].has("effects"):
				if dialog_data.data[item_id].effects.has(effect_data.code):
					var saved_effect = dialog_data.data[item_id].effects[effect_data.code]
					var removed = false
					
					if typeof(saved_effect) == TYPE_DICTIONARY:
						#print("Prepare deleting dictionary type")
						var found_index = -1
						for i in saved_effect.keys():
							# Comparing dic is a pain, even if the value is same it still detected not same
							#if saved_effect[i] == effect_data.value:
							#	found_index = i
							if typeof(effect_data.value) == TYPE_ARRAY:
								if var2str(saved_effect[i]) == var2str(effect_data.value):
									found_index = i
							else:
								if saved_effect[i] == effect_data.value:
									found_index = i
						if found_index >= 0:
							dialog_data.data[item_id].effects[effect_data.code].erase(found_index)
							removed = true
						
						if dialog_data.data[item_id].effects[effect_data.code].empty():
							dialog_data.data[item_id].effects.erase(effect_data.code)
						else:
							if dialog_data.data[item_id].effects[effect_data.code].size() == 1:
								var val_id = dialog_data.data[item_id].effects[effect_data.code].keys()[0]
								var val = dialog_data.data[item_id].effects[effect_data.code][val_id]
								dialog_data.data[item_id].effects[effect_data.code] = val
					else:
						#print("Prepare deleting non dictionary type")
						#print("saved_effect = %s" %str(saved_effect))
						#print("effect_data.value = %s" %str(effect_data.value))
						# Comparing dic is a pain, even if the value is same it still detected not same
						if typeof(saved_effect) == TYPE_ARRAY:
							if var2str(saved_effect) == var2str(effect_data.value):
								dialog_data.data[item_id].effects.erase(effect_data.code)
								removed = true
						else:
							if saved_effect == effect_data.value:
								dialog_data.data[item_id].effects.erase(effect_data.code)
								removed = true
					if dialog_data.data[item_id].effects.empty():
						dialog_data.data[item_id].erase("effects")
					if dialog_data.data[item_id].empty():
						dialog_data.data.erase(item_id)
					if removed:
						save_dialog_data()
	EffectOptions.hide()
	EffectTrigger.queue_free()
	# if saved to data, remove it too.

func open_chapter_add():
	ChapterAddDialog.set_as_minsize()
	ChapterAddDialog.get_ok().text = "Update"
	ChapterList.clear()
	LangList.hide()
	chapter_select = -1
	for i in ChapterData.data.size():
		add_chapter()
	ChapterAddDialog.popup_centered()

func confirm_chapter_add():
	if ChapterData == null:
		return
	var TransDic:Dictionary = get_config().get_value("settings","translations",{})
	var dialog_id = 0
	var prev_chapter = current_chapter
	for i in ChapterList.get_item_count():
		if not TransDic.empty():
			if TransDic.has(i):
				if TransDic[i].size() == Languages.size():
					var DRange = Vector2(dialog_id,-1)
					var Trans = load(TransDic[i][Languages[0]])
					while Trans.get_message("dialog_%s" %dialog_id) != "":
						dialog_id += 1
					DRange.y = (dialog_id - 1)
					var cur_choice = 0
					var choice_count = 0
					for key in ChapterData.data.keys():
						cur_choice += ChapterData.data[key]["choice"]
					while Trans.get_message("choice_%s_0" %cur_choice) != "":
						choice_count += 1
						cur_choice += 1
					print("AddData")
					ChapterData.data[i] = {}
					ChapterData.data[i]["range"] = DRange
					ChapterData.data[i]["choice"] = choice_count
					var data_path = ProjectSettings.get_setting(data_setting)
					var save_path = data_path.get_base_dir().plus_file("DialogChapter%s.tres" %i)
					ChapterData.data[i]["data_path"] = save_path
					var NewRes = DictionaryData.new()
					ResourceSaver.save(save_path,NewRes)
					ResourceSaver.save(data_path,ChapterData)
					current_chapter = i
	if prev_chapter != current_chapter:
		update_chapter(current_chapter,true)
		#init_items()
		#update_items()

func get_dialog_data_path(chapter_id):
	var data_path = ProjectSettings.get_setting(data_setting)
	var save_path = data_path.get_base_dir().plus_file("DialogChapter%s.tres" %chapter_id)
	return save_path

func get_dialog_data(update = false):
	if DialogData == null:
		update = true
	if update:
		var data_path = get_dialog_data_path(current_chapter)
		DialogData = load(data_path)
		if DialogData == null:
			DialogData = DictionaryData.new()
	return DialogData

func save_dialog_data():
	var save_path = get_dialog_data_path(current_chapter)
	var Data = get_dialog_data()
	ResourceSaver.save(save_path,Data)

func get_chapter_translations(update = false):
	if Translations.empty():
		update = true
	if update:
		Translations.clear()
		var ChapDic:Dictionary = get_config().get_value("settings","translations",{})
		if ChapDic.has(current_chapter):
			if ChapDic[current_chapter].size() == Languages.size():
				for i in Languages.size():
					var TransPaths = ChapDic[current_chapter]
					#print("Loading Trans for ch %s -> %s" %[current_chapter,TransPaths])
					var i_code = Languages[i]
					Translations.append(load(TransPaths[i_code]))
	return Translations

func update_chapter(num,update_item = false):
	print("Update chapter update item ? %s" %update_item)
	current_chapter = int(num)
	if int(current_chapter) != int(get_config().get_value("settings","current_chapter",-1)):
		update_config("settings","current_chapter",num)
	if num < 0:
		return
	print("Update chapter update item yyy ? %s" %update_item)
	if not ChapterChanger.editable:
		ChapterChanger.editable = true
		ChapterChanger.min_value = 0
	if ChapterChanger.max_value < num:
		ChapterChanger.max_value = num
	print("Update chapter update item xxx ? %s" %update_item)
	get_dialog_data(true)
	
	get_chapter_translations(true)
	print("Update chapter update item before check ? %s" %update_item)
	if update_item:
		print("Update chapter init item, update item -> %s" %update_item)
		init_items()
		update_items(false)
	
	update_chapter_title(num)
	
	var chapter_range = ChapterData.data[current_chapter]["range"]
	var line_count = chapter_range.y - chapter_range.x
	LineLabel.text = str(int(line_count) + 1)
	
	var chapter_choice = ChapterData.data[current_chapter]["choice"]
	ChoiceLabel.text = str(chapter_choice)
	
	ChapterChanger.value = num

func update_chapter_title(num):
	#var TransDic = get_config().get_value("settings","translations",{})
	#var lang_code = Languages[current_lang]
	#var Trans:PHashTranslation = load(TransDic[int(num)][lang_code])
	var Trans:PHashTranslation = get_chapter_translations()[current_lang]
	var ctitle = Trans.get_message("chapter_title_%s" %int(num))
	#print("Change chapter %s title -> %s" %[num,ctitle])
	#var txt = Trans.get_message("dialog_%s" %int(100))
	#if txt == "":
	#	txt = Trans.get_message("dialog_%s" %int(300))
	#print("   -> %s" %txt)
	ChapterTitle.text = ctitle

func add_chapter():
	var item_number = ChapterList.get_item_count()
	ChapterList.add_item("Chapter %s" %item_number)

func change_chapter_item(index):
	LangList.show()
	chapter_select = index
	var TransDic:Dictionary = get_config().get_value("settings","translations",{})
	if TransDic.empty():
		return
	if TransDic.has(index):
		for i in LangList.get_child_count():
			if TransDic[index].has(Languages[i]):
				LangList.get_child(i).get_node("List/Path").text = TransDic[index][Languages[i]]

func open_add_chapter_trans(id):
	print("open_add_chapter_trans %s, %s" %[id,chapter_select])
	if chapter_select < 0:
		return
	chapter_select_lang = id
	#TransFilesDialog.set_as_minsize()
	TransFilesDialog.popup_centered_minsize()

func translation_files_choosen(path):
	print("translation_files_choosen %s, %s" %[chapter_select,chapter_select_lang])
	if chapter_select < 0:
		return
	var TransDic:Dictionary = get_config().get_value("settings","translations",{})
	if not TransDic.has(chapter_select):
		TransDic[chapter_select] = {}
	TransDic[chapter_select][Languages[chapter_select_lang]] = path
	LangList.get_child(chapter_select_lang).get_node("List/Path").text = path
	update_config("settings","translations",TransDic)

func open_choose_data():
	#ChooseDataDialog.get_parent().visible = true
	#ChooseDataDialog.popup()
	#TransFilesDialog.set_as_minsize()
	ChooseDataDialog.popup_centered_minsize()

func data_file_choosen(path):
	ProjectSettings.set_setting(data_setting, path)
	ProjectSettings.save()
	FileLabel.text = path

func reload_dialog():
	#Translations = []
	Config = null
	var translations:PoolStringArray
	translations = get_config().get_value("settings","translations",PoolStringArray([]))
	if not translations.empty():
		TransFilesDialog.current_dir = translations[0].get_base_dir()
		var text = "Files:\n"
		for path in translations:
			var phash = load(path)
			if phash is PHashTranslation:
				#print("Added %s to translation table" %path)
				#Translations.append(phash)
				pass
			text += path + "\n"
		#TransPickBtn.hint_tooltip = text
	update_items()

