tool
extends PanelContainer

enum NodeType {
	Dialog,
	Choice,
	End,
	Branch,
	Operation
}

const VarItem = preload("res://addons/route_editor/VarItem.tscn")
const Nodes = [
	preload("res://addons/route_editor/NodeDialog.tscn"),
	preload("res://addons/route_editor/NodeChoice.tscn"),
	preload("res://addons/route_editor/NodeEnd.tscn"),
	preload("res://addons/route_editor/NodeBranch.tscn"),
	preload("res://addons/route_editor/NodeOperation.tscn")
]

onready var GraphPopup = $List/Editor/List/RouteGraph/GraphPopup
onready var RouteGraph = $List/Editor/List/RouteGraph
onready var VarParent = $List/Editor/List/VarPanel/Vars
onready var StartNode = $List/Editor/List/RouteGraph/Start
onready var MoveTimer = $MoveOffsetTimer

var Config:ConfigFile = null
var RouteData:DictionaryData = null
var Variables = []
var Langs = []

var config_path = "res://addons/route_editor/route_editor_config.cfg"
var dialog_setting = "vn/gameplay/data/chapters"
var route_setting = "vn/gameplay/data/route"
var scroll_offset = null
var dialog_indexes = Vector2(0,0)
var choice = 0

# Called when the node enters the scene tree for the first time.
func _ready():
	init_graph()
	init_langs()
	init_dialog_data()
	init_graph_option()
	init_graph_layout()
	init_route_data()
	

func get_config() -> ConfigFile:
	if Config == null:
		Config = ConfigFile.new()
		Config.load(config_path)
	return Config

func update_config(section,key,value):
	var Conf:ConfigFile = get_config()
	Conf.set_value(section,key,value)
	Conf.save(config_path)

func init_langs():
	var TranlationList = ProjectSettings.get_setting("locale/translations")
	for path in TranlationList:
		var number = int(path)
		if number > 0:
			Langs.append(load(path))
	#print("Lang detected: %s" %Langs.size())

func init_dialog_data():
	var d_data_path = ProjectSettings.get_setting(dialog_setting)
	var DialogData = load(d_data_path)
	for key in DialogData.data.keys():
		var c_range = DialogData.data[key]["range"]
		if c_range.x < dialog_indexes.x:
			dialog_indexes.x = c_range.x
		if c_range.y > dialog_indexes.y:
			dialog_indexes.y = c_range.y
		choice += DialogData.data[key]["choice"]

func init_graph_option():
	GraphPopup.set_item_disabled(1, true if choice == 0 else false)

func init_route_data():
	print("Init route data")
	StartNode.connect("dragged",self,"node_moved",[StartNode])
	var r_data_path = ProjectSettings.get_setting(route_setting)
	RouteData = load(r_data_path)
	
	#ADD START IF NOT IN THE DATA
	var StartNode = RouteGraph.get_node("Start")
	if not RouteData.data.has("nodes"):
		RouteData.data.nodes = {}
	if not RouteData.data.nodes.has("Start"):
		RouteData.data.nodes["Start"] = {}
	if not RouteData.data.nodes["Start"].has("offset"):
		RouteData.data.nodes["Start"].offset = StartNode.offset
		save_route_data()
	
	#INIT VARIABLES
	if RouteData.data.has("variables"):
		var vars = RouteData.data.variables
		for i in vars.size():
			add_var(vars[i],false)
	
	#INIT NODES
	if RouteData.data.has("nodes"):
		for node_name in RouteData.data.nodes.keys():
			var c_node_data = RouteData.data.nodes[node_name]
			var CNode = null
			if RouteGraph.has_node(node_name):
				CNode = RouteGraph.get_node(node_name)
				if c_node_data.has("offset"):
					CNode.offset = c_node_data.offset
			else:
				#print(c_node_data)
				CNode = add_node(c_node_data["type"],c_node_data.offset,false)
			if c_node_data.has("value"):
				#print(CNode.name)
				set_node_value(CNode,c_node_data.value)
	
	#INIT CONNECTIONS
	if RouteData.data.has("connections"):
		var connections = RouteData.data.connections
		for i in connections.size():
			connect_node(connections[i])

func save_route_data():
	var r_data_path = ProjectSettings.get_setting(route_setting)
	ResourceSaver.save(r_data_path,RouteData)

func init_graph():
	RouteGraph.add_valid_connection_type(0,0)

func init_graph_layout():
	yield(get_tree(),"idle_frame")
	#RouteGraph.scroll_offset = Vector2(-500,-150)
	RouteGraph.scroll_offset = get_config().get_value("settings","scroll_offset",Vector2(-500,-150))
	RouteGraph.connect("scroll_offset_changed",self,"graph_offset_updated")

func graph_offset_updated(offset):
	#print("Offset updated to %s" %offset)
	scroll_offset = offset
	MoveTimer.start()

func save_offset():
	scroll_offset = scroll_offset.round()
	RouteGraph.scroll_offset = scroll_offset
	update_config("settings","scroll_offset",scroll_offset)

func graph_connection_request(from, from_slot, to, to_slot):
	print("Connecting %s(%s) -> %s(%s)" %[from,from_slot,to,to_slot])
	RouteGraph.connect_node(from, from_slot, to, to_slot)
	RouteData.data.connections = RouteGraph.get_connection_list()
	save_route_data()

func graph_disconnection_request(from, from_slot, to, to_slot):
	print("Disconnecting %s(%s) -> %s(%s)" %[from,from_slot,to,to_slot])
	RouteGraph.disconnect_node(from, from_slot, to, to_slot)
	RouteData.data.connections = RouteGraph.get_connection_list()
	save_route_data()

func graph_input(event):
	if event is InputEventMouseButton:
		#if event.button_index == BUTTON_RIGHT:
		if Input.is_mouse_button_pressed(BUTTON_RIGHT):
			var mouse_pos = GraphPopup.get_global_mouse_position()
			GraphPopup.rect_global_position = mouse_pos
			GraphPopup.popup()

func popup_id_pressed(id):
	var mouse_pos = RouteGraph.get_local_mouse_position()
	mouse_pos.y = 0
	mouse_pos.x += RouteGraph.scroll_offset.x
	add_node(id,mouse_pos)

func add_node(type,position,save = true):
	var NewNode:GraphNode
	var node_pos = position
	#if save:
	#	node_pos.x += RouteGraph.scroll_offset.x
	match type:
		NodeType.Choice:
			NewNode = add_choice_node(node_pos)
		NodeType.Dialog:
			NewNode = add_dialog_node(node_pos)
		NodeType.End:
			NewNode = add_end_node(node_pos)
		NodeType.Branch:
			NewNode = add_branch_node(node_pos)
		NodeType.Operation:
			NewNode = add_operation_node(node_pos)
	
	NewNode.rect_size = Vector2(10,10)
	NewNode.connect("close_request",self,"node_deleted",[NewNode])
	
	if save:
		if not RouteData.data.has("nodes"):
			RouteData.data["nodes"] = {}
		if not RouteData.data.has(NewNode.name):
			RouteData.data.nodes[NewNode.name] = {}
		RouteData.data.nodes[NewNode.name].offset = position
		RouteData.data.nodes[NewNode.name].type = int(NewNode.editor_description)
		save_route_data()
	return NewNode

func add_dialog_node(position):
	var NewNode = Nodes[NodeType.Dialog].instance()
	NewNode.get_node("Refresh").connect("pressed",self,"reload_dialog_node",[NewNode])
	var StartSpin:SpinBox = NewNode.get_node("Start/Value")
	var EndSpin:SpinBox = NewNode.get_node("End/Value")
	StartSpin.min_value = dialog_indexes.x
	StartSpin.max_value = dialog_indexes.y
	EndSpin.min_value = dialog_indexes.x
	EndSpin.max_value = dialog_indexes.y
	NewNode.connect("dragged",self,"node_moved",[NewNode])
	NewNode.offset = position
	RouteGraph.add_child(NewNode)
	return NewNode

func reload_dialog_node(DNode,save = true):
	var StartLabel:Label = DNode.get_node("Start/Label")
	var EndLabel:Label = DNode.get_node("End/Label")
	var StartSpin:SpinBox = DNode.get_node("Start/Value")
	var EndSpin:SpinBox = DNode.get_node("End/Value")
	var start_text = ""
	var end_text = ""
	for i in Langs.size():
		if start_text == "":
			start_text = Langs[i].get_message("dialog_%s" %StartSpin.value)
		if end_text == "":
			end_text = Langs[i].get_message("dialog_%s" %EndSpin.value)
		if start_text != "" and end_text != "":
			break
	if start_text != "":
		StartLabel.hint_tooltip = start_text
	if end_text != "":
		EndLabel.hint_tooltip = end_text
	#print("Reload dialog %s" %DNode.name)
	if save:
		#print("Saving dialog %s" %DNode.name)
		var value = Vector2(StartSpin.value,EndSpin.value)
		if not RouteData.data.has("nodes"):
			RouteData.data["nodes"] = {}
		if not RouteData.data.nodes.has(DNode.name):
			RouteData.data.nodes[DNode.name] = {}
		var s_value = null
		if RouteData.data.nodes[DNode.name].has("value"):
			s_value = RouteData.data.nodes[DNode.name].value
		if s_value != null:
			if value != s_value:
				#print("Saving dialog %s" %DNode.name)
				RouteData.data.nodes[DNode.name].value = value
				save_route_data()
		else:
			RouteData.data.nodes[DNode.name].value = value
			save_route_data()
		

func add_choice_node(position):
	var NewNode = Nodes[NodeType.Choice].instance()
	var ChoiceSpin:SpinBox = NewNode.get_node("QID/ChoiceId")
	ChoiceSpin.min_value = 0
	ChoiceSpin.max_value = float(choice - 1)
	ChoiceSpin.value = -1
	ChoiceSpin.connect("value_changed",self,"change_choice",[NewNode])
	NewNode.connect("dragged",self,"node_moved",[NewNode])
	NewNode.offset = position
	RouteGraph.add_child(NewNode)
	return NewNode

func change_choice(index,CNode:GraphNode,save = true):
	var CSpin:SpinBox = CNode.get_node("QID/ChoiceId")
	var idx = int(index)
	if index < 0:
		CSpin.allow_lesser = false
		CSpin.value = index
	else:
		var options = []
		for i in Langs.size():
			var op_i = []
			var op_idx = 0
			var op_code = "choice_%s_%s" %[idx,op_idx]
			var op_text = Langs[i].get_message(op_code)
			while op_text != "":
				op_i.append(op_text)
				op_idx += 1
				op_code = "choice_%s_%s" %[idx,op_idx]
				op_text = Langs[i].get_message(op_code)
			if not op_i.empty():
				options = op_i
				break
		if CNode.get_child_count() > 2:
			var Childs = []
			for i in CNode.get_child_count():
				Childs.append(CNode.get_child(i))
			Childs.pop_front()
			Childs.pop_front()
			for Child in Childs:
				Child.free()
		for i in options.size():
			var NewChoice = Label.new()
			NewChoice.text = "Option %s" %(i+1)
			NewChoice.hint_tooltip = options[i]
			NewChoice.mouse_filter = Control.MOUSE_FILTER_STOP
			NewChoice.name = "Option%s" %(i+1)
			NewChoice.align = Label.ALIGN_RIGHT
			NewChoice.size_flags_horizontal = SIZE_SHRINK_END
			CNode.add_child(NewChoice)
			CNode.set("slot/%s/right_enabled" %(i+2),true)
		if save:
			var value = float(idx)
			if not RouteData.data.has("nodes"):
				RouteData.data["nodes"] = {}
			if not RouteData.data.nodes.has(CNode.name):
				RouteData.data.nodes[CNode.name] = {}
			var s_value = null
			if RouteData.data.nodes[CNode.name].has("value"):
				s_value = RouteData.data.nodes[CNode.name].value
			if s_value != null:
				if value != s_value:
					#print("Saving dialog %s" %DNode.name)
					RouteData.data.nodes[CNode.name].value = value
					save_route_data()
			else:
				RouteData.data.nodes[CNode.name].value = value
				save_route_data()

func add_end_node(position):
	var NewNode = Nodes[NodeType.End].instance()
	NewNode.connect("dragged",self,"node_moved",[NewNode])
	NewNode.offset = position
	RouteGraph.add_child(NewNode)
	return NewNode

func add_branch_node(position):
	var NewNode = Nodes[NodeType.Branch].instance()
	return NewNode

func add_operation_node(position):
	var NewNode = Nodes[NodeType.Operation].instance()
	var RouteOption:OptionButton = NewNode.get_node("List/Route")
	var SaveBtn:Button = NewNode.get_node("List/Save")
	SaveBtn.connect("pressed",self,"save_operation",[NewNode])
	for i in Variables.size():
		RouteOption.add_item(Variables[i])
	NewNode.connect("dragged",self,"node_moved",[NewNode])
	NewNode.offset = position
	RouteGraph.add_child(NewNode)
	return NewNode

func save_operation(ONode):
	var SaveBtn:Button = ONode.get_node("List/Save")
	var RouteOption:OptionButton = ONode.get_node("List/Route")
	var OperatorOption:OptionButton = ONode.get_node("List/Operator")
	var ValueSpin:SpinBox = ONode.get_node("List/Value")
	var expr = "%s %s %s" %[RouteOption.text,OperatorOption.text,ValueSpin.value]
	SaveBtn.hint_tooltip = expr
	if not RouteData.data.has("nodes"):
		RouteData.data["nodes"] = {}
	if not RouteData.data.nodes.has(ONode.name):
		RouteData.data.nodes[ONode.name] = {}
	var s_value = null
	if RouteData.data.nodes[ONode.name].has("value"):
		s_value = RouteData.data.nodes[ONode.name].value
	if s_value != null:
		if expr != s_value:
			#print("Saving dialog %s" %DNode.name)
			RouteData.data.nodes[ONode.name].value = expr
			save_route_data()
	else:
		RouteData.data.nodes[ONode.name].value = expr
		save_route_data()

func set_node_value(CNode,value):
	var type = null
	if CNode.editor_description == "":
		return
	else:
		type = int(CNode.editor_description)
	match type:
		NodeType.Choice:
			var ChoiceSpin:SpinBox = CNode.get_node("QID/ChoiceId")
			ChoiceSpin.value = value
		NodeType.Dialog:
			var StartSpin:SpinBox = CNode.get_node("Start/Value")
			var EndSpin:SpinBox = CNode.get_node("End/Value")
			StartSpin.value = value.x
			EndSpin.value = value.y
			reload_dialog_node(CNode,false)
		NodeType.End:
			pass
		NodeType.Branch:
			pass
		NodeType.Operation:
			var SaveBtn:Button = CNode.get_node("List/Save")
			var RouteOption:OptionButton = CNode.get_node("List/Route")
			var OperatorOption:OptionButton = CNode.get_node("List/Operator")
			var ValueSpin:SpinBox = CNode.get_node("List/Value")
			var values = value.split(" ")
			ValueSpin.value = float(values[2])
			var oper = 0
			for i in OperatorOption.get_item_count():
				var cur_op_txt = OperatorOption.get_item_text(i)
				if cur_op_txt == values[1]:
					OperatorOption.select(i)
					break
			var route = Variables.find(values[0])
			if route < 0:
				route = 0
			RouteOption.select(route)
			SaveBtn.hint_tooltip = value

func node_moved(from,to,GNode):
	#print("Node %s moved from %s to %s, offset %s" %[GNode.name,from,to,GNode.offset])
	#var 
	var save = false
	if not RouteData.data.has("nodes"):
		RouteData.data.nodes = {}
		if not RouteData.data["nodes"].has(GNode.name):
			RouteData.data.nodes[GNode.name] = {}
	if not RouteData.data.nodes[GNode.name].has("offset"):
		RouteData.data.nodes[GNode.name]["offset"] = to
		save = true
	else:
		var saved_offset = RouteData.data.nodes[GNode.name]["offset"]
		if saved_offset != to:
			RouteData.data.nodes[GNode.name]["offset"] = to
			save = true
	if save:
		save_route_data()
	

func node_deleted(node):
	var ConList = RouteGraph.get_connection_list()
	for Connection in ConList:
		if Connection.from == node.name:
			disconnect_node(Connection)
		if Connection.to == node.name:
			disconnect_node(Connection)
	var save = false
	if RouteData.data.has("nodes"):
		if RouteData.data.nodes.has(node.name):
			RouteData.data.nodes.erase(node.name)
			save = true
	#ALSO SAVE CONNECTIONS!!
	if save:
		RouteData.data.connections = RouteGraph.get_connection_list()
		save_route_data()

func disconnect_node(Dic):
	RouteGraph.disconnect_node(Dic.from, Dic.from_port, Dic.to, Dic.to_port)

func connect_node(Dic):
	RouteGraph.connect_node(Dic.from, Dic.from_port, Dic.to, Dic.to_port)

func add_var(content = "",save = true):
	var NewVar = VarItem.instance()
	var child_count = VarParent.get_child_count()
	NewVar.name = "Var%s" %str(child_count + 1)
	if content != "":
		NewVar.get_node("LineEdit").text = content
	var EditButton = NewVar.get_node("EditButton")
	var LineEditor = NewVar.get_node("LineEdit")
	EditButton.connect("toggled",self,"focus_line",[LineEditor])
	NewVar.get_node("EraseButton").connect("pressed",self,"remove_var",[child_count])
	LineEditor.connect("text_entered",self,"edit_var",[child_count,EditButton])
	Variables.append(content)
	VarParent.add_child(NewVar)
	if save:
		RouteData.data["variables"] = Variables
		save_route_data()

func focus_line(state,LineEditor:LineEdit):
	if state:
		LineEditor.grab_focus()

func edit_var(value,index,EditButton:Button = null):
	#print("entered value %s" %value)
	Variables[index] = value
	if EditButton != null:
		EditButton.get_focus_owner().release_focus()
		EditButton.pressed = false
	RouteData.data["variables"] = Variables
	save_route_data()

func remove_var(index):
	Variables.remove(index)
	RouteData.data["variables"] = Variables
	save_route_data()

